# self file was generated automatically by the Snowball to Python interpreter

from .basestemmer import BaseStemmer
from .among import Among


class RussianStemmer(BaseStemmer):
    '''
    self class was automatically generated by a Snowball to Python interpreter
    It implements the stemming algorithm defined by a snowball script.
    '''
    serialVersionUID = 1

    a_0 = [
        Among(u"\u0432", -1, 1),
        Among(u"\u0438\u0432", 0, 2),
        Among(u"\u044B\u0432", 0, 2),
        Among(u"\u0432\u0448\u0438", -1, 1),
        Among(u"\u0438\u0432\u0448\u0438", 3, 2),
        Among(u"\u044B\u0432\u0448\u0438", 3, 2),
        Among(u"\u0432\u0448\u0438\u0441\u044C", -1, 1),
        Among(u"\u0438\u0432\u0448\u0438\u0441\u044C", 6, 2),
        Among(u"\u044B\u0432\u0448\u0438\u0441\u044C", 6, 2)
    ]

    a_1 = [
        Among(u"\u0435\u0435", -1, 1),
        Among(u"\u0438\u0435", -1, 1),
        Among(u"\u043E\u0435", -1, 1),
        Among(u"\u044B\u0435", -1, 1),
        Among(u"\u0438\u043C\u0438", -1, 1),
        Among(u"\u044B\u043C\u0438", -1, 1),
        Among(u"\u0435\u0439", -1, 1),
        Among(u"\u0438\u0439", -1, 1),
        Among(u"\u043E\u0439", -1, 1),
        Among(u"\u044B\u0439", -1, 1),
        Among(u"\u0435\u043C", -1, 1),
        Among(u"\u0438\u043C", -1, 1),
        Among(u"\u043E\u043C", -1, 1),
        Among(u"\u044B\u043C", -1, 1),
        Among(u"\u0435\u0433\u043E", -1, 1),
        Among(u"\u043E\u0433\u043E", -1, 1),
        Among(u"\u0435\u043C\u0443", -1, 1),
        Among(u"\u043E\u043C\u0443", -1, 1),
        Among(u"\u0438\u0445", -1, 1),
        Among(u"\u044B\u0445", -1, 1),
        Among(u"\u0435\u044E", -1, 1),
        Among(u"\u043E\u044E", -1, 1),
        Among(u"\u0443\u044E", -1, 1),
        Among(u"\u044E\u044E", -1, 1),
        Among(u"\u0430\u044F", -1, 1),
        Among(u"\u044F\u044F", -1, 1)
    ]

    a_2 = [
        Among(u"\u0435\u043C", -1, 1),
        Among(u"\u043D\u043D", -1, 1),
        Among(u"\u0432\u0448", -1, 1),
        Among(u"\u0438\u0432\u0448", 2, 2),
        Among(u"\u044B\u0432\u0448", 2, 2),
        Among(u"\u0449", -1, 1),
        Among(u"\u044E\u0449", 5, 1),
        Among(u"\u0443\u044E\u0449", 6, 2)
    ]

    a_3 = [
        Among(u"\u0441\u044C", -1, 1),
        Among(u"\u0441\u044F", -1, 1)
    ]

    a_4 = [
        Among(u"\u043B\u0430", -1, 1),
        Among(u"\u0438\u043B\u0430", 0, 2),
        Among(u"\u044B\u043B\u0430", 0, 2),
        Among(u"\u043D\u0430", -1, 1),
        Among(u"\u0435\u043D\u0430", 3, 2),
        Among(u"\u0435\u0442\u0435", -1, 1),
        Among(u"\u0438\u0442\u0435", -1, 2),
        Among(u"\u0439\u0442\u0435", -1, 1),
        Among(u"\u0435\u0439\u0442\u0435", 7, 2),
        Among(u"\u0443\u0439\u0442\u0435", 7, 2),
        Among(u"\u043B\u0438", -1, 1),
        Among(u"\u0438\u043B\u0438", 10, 2),
        Among(u"\u044B\u043B\u0438", 10, 2),
        Among(u"\u0439", -1, 1),
        Among(u"\u0435\u0439", 13, 2),
        Among(u"\u0443\u0439", 13, 2),
        Among(u"\u043B", -1, 1),
        Among(u"\u0438\u043B", 16, 2),
        Among(u"\u044B\u043B", 16, 2),
        Among(u"\u0435\u043C", -1, 1),
        Among(u"\u0438\u043C", -1, 2),
        Among(u"\u044B\u043C", -1, 2),
        Among(u"\u043D", -1, 1),
        Among(u"\u0435\u043D", 22, 2),
        Among(u"\u043B\u043E", -1, 1),
        Among(u"\u0438\u043B\u043E", 24, 2),
        Among(u"\u044B\u043B\u043E", 24, 2),
        Among(u"\u043D\u043E", -1, 1),
        Among(u"\u0435\u043D\u043E", 27, 2),
        Among(u"\u043D\u043D\u043E", 27, 1),
        Among(u"\u0435\u0442", -1, 1),
        Among(u"\u0443\u0435\u0442", 30, 2),
        Among(u"\u0438\u0442", -1, 2),
        Among(u"\u044B\u0442", -1, 2),
        Among(u"\u044E\u0442", -1, 1),
        Among(u"\u0443\u044E\u0442", 34, 2),
        Among(u"\u044F\u0442", -1, 2),
        Among(u"\u043D\u044B", -1, 1),
        Among(u"\u0435\u043D\u044B", 37, 2),
        Among(u"\u0442\u044C", -1, 1),
        Among(u"\u0438\u0442\u044C", 39, 2),
        Among(u"\u044B\u0442\u044C", 39, 2),
        Among(u"\u0435\u0448\u044C", -1, 1),
        Among(u"\u0438\u0448\u044C", -1, 2),
        Among(u"\u044E", -1, 2),
        Among(u"\u0443\u044E", 44, 2)
    ]

    a_5 = [
        Among(u"\u0430", -1, 1),
        Among(u"\u0435\u0432", -1, 1),
        Among(u"\u043E\u0432", -1, 1),
        Among(u"\u0435", -1, 1),
        Among(u"\u0438\u0435", 3, 1),
        Among(u"\u044C\u0435", 3, 1),
        Among(u"\u0438", -1, 1),
        Among(u"\u0435\u0438", 6, 1),
        Among(u"\u0438\u0438", 6, 1),
        Among(u"\u0430\u043C\u0438", 6, 1),
        Among(u"\u044F\u043C\u0438", 6, 1),
        Among(u"\u0438\u044F\u043C\u0438", 10, 1),
        Among(u"\u0439", -1, 1),
        Among(u"\u0435\u0439", 12, 1),
        Among(u"\u0438\u0435\u0439", 13, 1),
        Among(u"\u0438\u0439", 12, 1),
        Among(u"\u043E\u0439", 12, 1),
        Among(u"\u0430\u043C", -1, 1),
        Among(u"\u0435\u043C", -1, 1),
        Among(u"\u0438\u0435\u043C", 18, 1),
        Among(u"\u043E\u043C", -1, 1),
        Among(u"\u044F\u043C", -1, 1),
        Among(u"\u0438\u044F\u043C", 21, 1),
        Among(u"\u043E", -1, 1),
        Among(u"\u0443", -1, 1),
        Among(u"\u0430\u0445", -1, 1),
        Among(u"\u044F\u0445", -1, 1),
        Among(u"\u0438\u044F\u0445", 26, 1),
        Among(u"\u044B", -1, 1),
        Among(u"\u044C", -1, 1),
        Among(u"\u044E", -1, 1),
        Among(u"\u0438\u044E", 30, 1),
        Among(u"\u044C\u044E", 30, 1),
        Among(u"\u044F", -1, 1),
        Among(u"\u0438\u044F", 33, 1),
        Among(u"\u044C\u044F", 33, 1)
    ]

    a_6 = [
        Among(u"\u043E\u0441\u0442", -1, 1),
        Among(u"\u043E\u0441\u0442\u044C", -1, 1)
    ]

    a_7 = [
        Among(u"\u0435\u0439\u0448\u0435", -1, 1),
        Among(u"\u043D", -1, 2),
        Among(u"\u0435\u0439\u0448", -1, 1),
        Among(u"\u044C", -1, 3)
    ]

    g_v = [33, 65, 8, 232]

    I_p2 = 0
    I_pV = 0

    def copy_from(self, other):
        self.I_p2 = other.I_p2
        self.I_pV = other.I_pV
        super.copy_from(other)
    

    def r_mark_regions(self):
        # (, line 57
        self.I_pV = self.limit;
        self.I_p2 = self.limit;
        # do, line 61
        v_1 = self.cursor
        try:
            # (, line 61
            # gopast, line 62
            try:
                while True:
                    try:
                        if not self.in_grouping(RussianStemmer.g_v, 1072, 1103):
                            raise lab2()
                        raise lab1()
                    except lab2: pass
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
            except lab1: pass
            # setmark pV, line 62
            self.I_pV = self.cursor
            # gopast, line 62
            try:
                while True:
                    try:
                        if not self.out_grouping(RussianStemmer.g_v, 1072, 1103):
                            raise lab4()
                        raise lab3()
                    except lab4: pass
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
            except lab3: pass
            # gopast, line 63
            try:
                while True:
                    try:
                        if not self.in_grouping(RussianStemmer.g_v, 1072, 1103):
                            raise lab6()
                        raise lab5()
                    except lab6: pass
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
            except lab5: pass
            # gopast, line 63
            try:
                while True:
                    try:
                        if not self.out_grouping(RussianStemmer.g_v, 1072, 1103):
                            raise lab8()
                        raise lab7()
                    except lab8: pass
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
            except lab7: pass
            # setmark p2, line 63
            self.I_p2 = self.cursor
        except lab0: pass
        self.cursor = v_1
        return True

    def r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def r_perfective_gerund(self):
        # (, line 71
        # [, line 72
        self.ket = self.cursor
        # substring, line 72
        among_var = self.find_among_b(RussianStemmer.a_0, 9)
        if among_var == 0:
            return False
        # ], line 72
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 76
            # or, line 76
            try:
                v_1 = self.limit - self.cursor
                try:
                    # literal, line 76
                    if not self.eq_s_b(1, u"\u0430"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                # literal, line 76
                if not self.eq_s_b(1, u"\u044F"):
                    return False
            except lab0: pass
            # delete, line 76
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 83
            # delete, line 83
            if not self.slice_del():
                return False

        return True

    def r_adjective(self):
        # (, line 87
        # [, line 88
        self.ket = self.cursor
        # substring, line 88
        among_var = self.find_among_b(RussianStemmer.a_1, 26)
        if among_var == 0:
            return False
        # ], line 88
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 97
            # delete, line 97
            if not self.slice_del():
                return False

        return True

    def r_adjectival(self):
        # (, line 101
        # call adjective, line 102
        if not self.r_adjective():
            return False
        # try, line 109
        v_1 = self.limit - self.cursor
        try:
            # (, line 109
            # [, line 110
            self.ket = self.cursor
            # substring, line 110
            among_var = self.find_among_b(RussianStemmer.a_2, 8)
            if among_var == 0:
                self.cursor = self.limit - v_1
                raise lab0()
            # ], line 110
            self.bra = self.cursor
            if among_var == 0:
                self.cursor = self.limit - v_1
                raise lab0()
            elif among_var == 1:
                # (, line 115
                # or, line 115
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        # literal, line 115
                        if not self.eq_s_b(1, u"\u0430"):
                            raise lab2()
                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_2
                    # literal, line 115
                    if not self.eq_s_b(1, u"\u044F"):
                        self.cursor = self.limit - v_1
                        raise lab0()
                except lab1: pass
                # delete, line 115
                if not self.slice_del():
                    return False

            elif among_var == 2:
                # (, line 122
                # delete, line 122
                if not self.slice_del():
                    return False

        except lab0: pass
        return True

    def r_reflexive(self):
        # (, line 128
        # [, line 129
        self.ket = self.cursor
        # substring, line 129
        among_var = self.find_among_b(RussianStemmer.a_3, 2)
        if among_var == 0:
            return False
        # ], line 129
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 132
            # delete, line 132
            if not self.slice_del():
                return False

        return True

    def r_verb(self):
        # (, line 136
        # [, line 137
        self.ket = self.cursor
        # substring, line 137
        among_var = self.find_among_b(RussianStemmer.a_4, 46)
        if among_var == 0:
            return False
        # ], line 137
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 143
            # or, line 143
            try:
                v_1 = self.limit - self.cursor
                try:
                    # literal, line 143
                    if not self.eq_s_b(1, u"\u0430"):
                        raise lab1()
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                # literal, line 143
                if not self.eq_s_b(1, u"\u044F"):
                    return False
            except lab0: pass
            # delete, line 143
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 151
            # delete, line 151
            if not self.slice_del():
                return False

        return True

    def r_noun(self):
        # (, line 159
        # [, line 160
        self.ket = self.cursor
        # substring, line 160
        among_var = self.find_among_b(RussianStemmer.a_5, 36)
        if among_var == 0:
            return False
        # ], line 160
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 167
            # delete, line 167
            if not self.slice_del():
                return False

        return True

    def r_derivational(self):
        # (, line 175
        # [, line 176
        self.ket = self.cursor
        # substring, line 176
        among_var = self.find_among_b(RussianStemmer.a_6, 2)
        if among_var == 0:
            return False
        # ], line 176
        self.bra = self.cursor
        # call R2, line 176
        if not self.r_R2():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 179
            # delete, line 179
            if not self.slice_del():
                return False

        return True

    def r_tidy_up(self):
        # (, line 183
        # [, line 184
        self.ket = self.cursor
        # substring, line 184
        among_var = self.find_among_b(RussianStemmer.a_7, 4)
        if among_var == 0:
            return False
        # ], line 184
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 188
            # delete, line 188
            if not self.slice_del():
                return False

            # [, line 189
            self.ket = self.cursor
            # literal, line 189
            if not self.eq_s_b(1, u"\u043D"):
                return False
            # ], line 189
            self.bra = self.cursor
            # literal, line 189
            if not self.eq_s_b(1, u"\u043D"):
                return False
            # delete, line 189
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 192
            # literal, line 192
            if not self.eq_s_b(1, u"\u043D"):
                return False
            # delete, line 192
            if not self.slice_del():
                return False

        elif among_var == 3:
            # (, line 194
            # delete, line 194
            if not self.slice_del():
                return False

        return True

    def _stem(self):
        # (, line 199
        # do, line 201
        v_1 = self.cursor
        try:
            # call mark_regions, line 201
            if not self.r_mark_regions():
                raise lab0()
        except lab0: pass
        self.cursor = v_1
        # backwards, line 202
        self.limit_backward = self.cursor
        self.cursor = self.limit
        # setlimit, line 202
        v_2 = self.limit - self.cursor
        # tomark, line 202
        if self.cursor < self.I_pV:
            return False
        self.cursor = self.I_pV
        v_3 = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit - v_2
        # (, line 202
        # do, line 203
        v_4 = self.limit - self.cursor
        try:
            # (, line 203
            # or, line 204
            try:
                v_5 = self.limit - self.cursor
                try:
                    # call perfective_gerund, line 204
                    if not self.r_perfective_gerund():
                        raise lab3()
                    raise lab2()
                except lab3: pass
                self.cursor = self.limit - v_5
                # (, line 205
                # try, line 205
                v_6 = self.limit - self.cursor
                try:
                    # call reflexive, line 205
                    if not self.r_reflexive():
                        self.cursor = self.limit - v_6
                        raise lab4()
                except lab4: pass
                # or, line 206
                try:
                    v_7 = self.limit - self.cursor
                    try:
                        # call adjectival, line 206
                        if not self.r_adjectival():
                            raise lab6()
                        raise lab5()
                    except lab6: pass
                    self.cursor = self.limit - v_7
                    try:
                        # call verb, line 206
                        if not self.r_verb():
                            raise lab7()
                        raise lab5()
                    except lab7: pass
                    self.cursor = self.limit - v_7
                    # call noun, line 206
                    if not self.r_noun():
                        raise lab1()
                except lab5: pass
            except lab2: pass
        except lab1: pass
        self.cursor = self.limit - v_4
        # try, line 209
        v_8 = self.limit - self.cursor
        try:
            # (, line 209
            # [, line 209
            self.ket = self.cursor
            # literal, line 209
            if not self.eq_s_b(1, u"\u0438"):
                self.cursor = self.limit - v_8
                raise lab8()
            # ], line 209
            self.bra = self.cursor
            # delete, line 209
            if not self.slice_del():
                return False

        except lab8: pass
        # do, line 212
        v_9 = self.limit - self.cursor
        try:
            # call derivational, line 212
            if not self.r_derivational():
                raise lab9()
        except lab9: pass
        self.cursor = self.limit - v_9
        # do, line 213
        v_10 = self.limit - self.cursor
        try:
            # call tidy_up, line 213
            if not self.r_tidy_up():
                raise lab10()
        except lab10: pass
        self.cursor = self.limit - v_10
        self.limit_backward = v_3
        self.cursor = self.limit_backward
        return True

    def equals(self, o):
        return isinstance(o, RussianStemmer)

    def hashCode(self):
        return hash("RussianStemmer")
class lab0(BaseException): pass
class lab1(BaseException): pass
class lab2(BaseException): pass
class lab3(BaseException): pass
class lab4(BaseException): pass
class lab5(BaseException): pass
class lab6(BaseException): pass
class lab7(BaseException): pass
class lab8(BaseException): pass
class lab9(BaseException): pass
class lab10(BaseException): pass



# self file was generated automatically by the Snowball to Python interpreter

from .basestemmer import BaseStemmer
from .among import Among


class PortugueseStemmer(BaseStemmer):
    '''
    self class was automatically generated by a Snowball to Python interpreter
    It implements the stemming algorithm defined by a snowball script.
    '''
    serialVersionUID = 1

    a_0 = [
        Among(u"", -1, 3),
        Among(u"\u00E3", 0, 1),
        Among(u"\u00F5", 0, 2)
    ]

    a_1 = [
        Among(u"", -1, 3),
        Among(u"a~", 0, 1),
        Among(u"o~", 0, 2)
    ]

    a_2 = [
        Among(u"ic", -1, -1),
        Among(u"ad", -1, -1),
        Among(u"os", -1, -1),
        Among(u"iv", -1, 1)
    ]

    a_3 = [
        Among(u"ante", -1, 1),
        Among(u"avel", -1, 1),
        Among(u"\u00EDvel", -1, 1)
    ]

    a_4 = [
        Among(u"ic", -1, 1),
        Among(u"abil", -1, 1),
        Among(u"iv", -1, 1)
    ]

    a_5 = [
        Among(u"ica", -1, 1),
        Among(u"\u00E2ncia", -1, 1),
        Among(u"\u00EAncia", -1, 4),
        Among(u"ira", -1, 9),
        Among(u"adora", -1, 1),
        Among(u"osa", -1, 1),
        Among(u"ista", -1, 1),
        Among(u"iva", -1, 8),
        Among(u"eza", -1, 1),
        Among(u"log\u00EDa", -1, 2),
        Among(u"idade", -1, 7),
        Among(u"ante", -1, 1),
        Among(u"mente", -1, 6),
        Among(u"amente", 12, 5),
        Among(u"\u00E1vel", -1, 1),
        Among(u"\u00EDvel", -1, 1),
        Among(u"uci\u00F3n", -1, 3),
        Among(u"ico", -1, 1),
        Among(u"ismo", -1, 1),
        Among(u"oso", -1, 1),
        Among(u"amento", -1, 1),
        Among(u"imento", -1, 1),
        Among(u"ivo", -1, 8),
        Among(u"a\u00E7a~o", -1, 1),
        Among(u"ador", -1, 1),
        Among(u"icas", -1, 1),
        Among(u"\u00EAncias", -1, 4),
        Among(u"iras", -1, 9),
        Among(u"adoras", -1, 1),
        Among(u"osas", -1, 1),
        Among(u"istas", -1, 1),
        Among(u"ivas", -1, 8),
        Among(u"ezas", -1, 1),
        Among(u"log\u00EDas", -1, 2),
        Among(u"idades", -1, 7),
        Among(u"uciones", -1, 3),
        Among(u"adores", -1, 1),
        Among(u"antes", -1, 1),
        Among(u"a\u00E7o~es", -1, 1),
        Among(u"icos", -1, 1),
        Among(u"ismos", -1, 1),
        Among(u"osos", -1, 1),
        Among(u"amentos", -1, 1),
        Among(u"imentos", -1, 1),
        Among(u"ivos", -1, 8)
    ]

    a_6 = [
        Among(u"ada", -1, 1),
        Among(u"ida", -1, 1),
        Among(u"ia", -1, 1),
        Among(u"aria", 2, 1),
        Among(u"eria", 2, 1),
        Among(u"iria", 2, 1),
        Among(u"ara", -1, 1),
        Among(u"era", -1, 1),
        Among(u"ira", -1, 1),
        Among(u"ava", -1, 1),
        Among(u"asse", -1, 1),
        Among(u"esse", -1, 1),
        Among(u"isse", -1, 1),
        Among(u"aste", -1, 1),
        Among(u"este", -1, 1),
        Among(u"iste", -1, 1),
        Among(u"ei", -1, 1),
        Among(u"arei", 16, 1),
        Among(u"erei", 16, 1),
        Among(u"irei", 16, 1),
        Among(u"am", -1, 1),
        Among(u"iam", 20, 1),
        Among(u"ariam", 21, 1),
        Among(u"eriam", 21, 1),
        Among(u"iriam", 21, 1),
        Among(u"aram", 20, 1),
        Among(u"eram", 20, 1),
        Among(u"iram", 20, 1),
        Among(u"avam", 20, 1),
        Among(u"em", -1, 1),
        Among(u"arem", 29, 1),
        Among(u"erem", 29, 1),
        Among(u"irem", 29, 1),
        Among(u"assem", 29, 1),
        Among(u"essem", 29, 1),
        Among(u"issem", 29, 1),
        Among(u"ado", -1, 1),
        Among(u"ido", -1, 1),
        Among(u"ando", -1, 1),
        Among(u"endo", -1, 1),
        Among(u"indo", -1, 1),
        Among(u"ara~o", -1, 1),
        Among(u"era~o", -1, 1),
        Among(u"ira~o", -1, 1),
        Among(u"ar", -1, 1),
        Among(u"er", -1, 1),
        Among(u"ir", -1, 1),
        Among(u"as", -1, 1),
        Among(u"adas", 47, 1),
        Among(u"idas", 47, 1),
        Among(u"ias", 47, 1),
        Among(u"arias", 50, 1),
        Among(u"erias", 50, 1),
        Among(u"irias", 50, 1),
        Among(u"aras", 47, 1),
        Among(u"eras", 47, 1),
        Among(u"iras", 47, 1),
        Among(u"avas", 47, 1),
        Among(u"es", -1, 1),
        Among(u"ardes", 58, 1),
        Among(u"erdes", 58, 1),
        Among(u"irdes", 58, 1),
        Among(u"ares", 58, 1),
        Among(u"eres", 58, 1),
        Among(u"ires", 58, 1),
        Among(u"asses", 58, 1),
        Among(u"esses", 58, 1),
        Among(u"isses", 58, 1),
        Among(u"astes", 58, 1),
        Among(u"estes", 58, 1),
        Among(u"istes", 58, 1),
        Among(u"is", -1, 1),
        Among(u"ais", 71, 1),
        Among(u"eis", 71, 1),
        Among(u"areis", 73, 1),
        Among(u"ereis", 73, 1),
        Among(u"ireis", 73, 1),
        Among(u"\u00E1reis", 73, 1),
        Among(u"\u00E9reis", 73, 1),
        Among(u"\u00EDreis", 73, 1),
        Among(u"\u00E1sseis", 73, 1),
        Among(u"\u00E9sseis", 73, 1),
        Among(u"\u00EDsseis", 73, 1),
        Among(u"\u00E1veis", 73, 1),
        Among(u"\u00EDeis", 73, 1),
        Among(u"ar\u00EDeis", 84, 1),
        Among(u"er\u00EDeis", 84, 1),
        Among(u"ir\u00EDeis", 84, 1),
        Among(u"ados", -1, 1),
        Among(u"idos", -1, 1),
        Among(u"amos", -1, 1),
        Among(u"\u00E1ramos", 90, 1),
        Among(u"\u00E9ramos", 90, 1),
        Among(u"\u00EDramos", 90, 1),
        Among(u"\u00E1vamos", 90, 1),
        Among(u"\u00EDamos", 90, 1),
        Among(u"ar\u00EDamos", 95, 1),
        Among(u"er\u00EDamos", 95, 1),
        Among(u"ir\u00EDamos", 95, 1),
        Among(u"emos", -1, 1),
        Among(u"aremos", 99, 1),
        Among(u"eremos", 99, 1),
        Among(u"iremos", 99, 1),
        Among(u"\u00E1ssemos", 99, 1),
        Among(u"\u00EAssemos", 99, 1),
        Among(u"\u00EDssemos", 99, 1),
        Among(u"imos", -1, 1),
        Among(u"armos", -1, 1),
        Among(u"ermos", -1, 1),
        Among(u"irmos", -1, 1),
        Among(u"\u00E1mos", -1, 1),
        Among(u"ar\u00E1s", -1, 1),
        Among(u"er\u00E1s", -1, 1),
        Among(u"ir\u00E1s", -1, 1),
        Among(u"eu", -1, 1),
        Among(u"iu", -1, 1),
        Among(u"ou", -1, 1),
        Among(u"ar\u00E1", -1, 1),
        Among(u"er\u00E1", -1, 1),
        Among(u"ir\u00E1", -1, 1)
    ]

    a_7 = [
        Among(u"a", -1, 1),
        Among(u"i", -1, 1),
        Among(u"o", -1, 1),
        Among(u"os", -1, 1),
        Among(u"\u00E1", -1, 1),
        Among(u"\u00ED", -1, 1),
        Among(u"\u00F3", -1, 1)
    ]

    a_8 = [
        Among(u"e", -1, 1),
        Among(u"\u00E7", -1, 2),
        Among(u"\u00E9", -1, 1),
        Among(u"\u00EA", -1, 1)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 19, 12, 2]

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def copy_from(self, other):
        self.I_p2 = other.I_p2
        self.I_p1 = other.I_p1
        self.I_pV = other.I_pV
        super.copy_from(other)
    

    def r_prelude(self):
        # repeat, line 36
        try:
            while True:
                try:
                    v_1 = self.cursor
                    try:
                        # (, line 36
                        # [, line 37
                        self.bra = self.cursor
                        # substring, line 37
                        among_var = self.find_among(PortugueseStemmer.a_0, 3)
                        if among_var == 0:
                            raise lab2()
                        # ], line 37
                        self.ket = self.cursor
                        if among_var == 0:
                            raise lab2()
                        elif among_var == 1:
                            # (, line 38
                            # <-, line 38
                            if not self.slice_from(u"a~"):
                                return False
                        elif among_var == 2:
                            # (, line 39
                            # <-, line 39
                            if not self.slice_from(u"o~"):
                                return False
                        elif among_var == 3:
                            # (, line 40
                            # next, line 40
                            if self.cursor >= self.limit:
                                raise lab2()
                            self.cursor += 1
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_1
                    raise lab0()
                except lab1: pass
        except lab0: pass
        return True

    def r_mark_regions(self):
        # (, line 44
        self.I_pV = self.limit;
        self.I_p1 = self.limit;
        self.I_p2 = self.limit;
        # do, line 50
        v_1 = self.cursor
        try:
            # (, line 50
            # or, line 52
            try:
                v_2 = self.cursor
                try:
                    # (, line 51
                    if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                        raise lab2()
                    # or, line 51
                    try:
                        v_3 = self.cursor
                        try:
                            # (, line 51
                            if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                                raise lab4()
                            # gopast, line 51
                            try:
                                while True:
                                    try:
                                        if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                                            raise lab6()
                                        raise lab5()
                                    except lab6: pass
                                    if self.cursor >= self.limit:
                                        raise lab4()
                                    self.cursor += 1
                            except lab5: pass
                            raise lab3()
                        except lab4: pass
                        self.cursor = v_3
                        # (, line 51
                        if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab2()
                        # gopast, line 51
                        try:
                            while True:
                                try:
                                    if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                                        raise lab8()
                                    raise lab7()
                                except lab8: pass
                                if self.cursor >= self.limit:
                                    raise lab2()
                                self.cursor += 1
                        except lab7: pass
                    except lab3: pass
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                # (, line 53
                if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                    raise lab0()
                # or, line 53
                try:
                    v_6 = self.cursor
                    try:
                        # (, line 53
                        if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab10()
                        # gopast, line 53
                        try:
                            while True:
                                try:
                                    if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                                        raise lab12()
                                    raise lab11()
                                except lab12: pass
                                if self.cursor >= self.limit:
                                    raise lab10()
                                self.cursor += 1
                        except lab11: pass
                        raise lab9()
                    except lab10: pass
                    self.cursor = v_6
                    # (, line 53
                    if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                        raise lab0()
                    # next, line 53
                    if self.cursor >= self.limit:
                        raise lab0()
                    self.cursor += 1
                except lab9: pass
            except lab1: pass
            # setmark pV, line 54
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        # do, line 56
        v_8 = self.cursor
        try:
            # (, line 56
            # gopast, line 57
            try:
                while True:
                    try:
                        if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab15()
                        raise lab14()
                    except lab15: pass
                    if self.cursor >= self.limit:
                        raise lab13()
                    self.cursor += 1
            except lab14: pass
            # gopast, line 57
            try:
                while True:
                    try:
                        if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab17()
                        raise lab16()
                    except lab17: pass
                    if self.cursor >= self.limit:
                        raise lab13()
                    self.cursor += 1
            except lab16: pass
            # setmark p1, line 57
            self.I_p1 = self.cursor
            # gopast, line 58
            try:
                while True:
                    try:
                        if not self.in_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab19()
                        raise lab18()
                    except lab19: pass
                    if self.cursor >= self.limit:
                        raise lab13()
                    self.cursor += 1
            except lab18: pass
            # gopast, line 58
            try:
                while True:
                    try:
                        if not self.out_grouping(PortugueseStemmer.g_v, 97, 250):
                            raise lab21()
                        raise lab20()
                    except lab21: pass
                    if self.cursor >= self.limit:
                        raise lab13()
                    self.cursor += 1
            except lab20: pass
            # setmark p2, line 58
            self.I_p2 = self.cursor
        except lab13: pass
        self.cursor = v_8
        return True

    def r_postlude(self):
        # repeat, line 62
        try:
            while True:
                try:
                    v_1 = self.cursor
                    try:
                        # (, line 62
                        # [, line 63
                        self.bra = self.cursor
                        # substring, line 63
                        among_var = self.find_among(PortugueseStemmer.a_1, 3)
                        if among_var == 0:
                            raise lab2()
                        # ], line 63
                        self.ket = self.cursor
                        if among_var == 0:
                            raise lab2()
                        elif among_var == 1:
                            # (, line 64
                            # <-, line 64
                            if not self.slice_from(u"\u00E3"):
                                return False
                        elif among_var == 2:
                            # (, line 65
                            # <-, line 65
                            if not self.slice_from(u"\u00F5"):
                                return False
                        elif among_var == 3:
                            # (, line 66
                            # next, line 66
                            if self.cursor >= self.limit:
                                raise lab2()
                            self.cursor += 1
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_1
                    raise lab0()
                except lab1: pass
        except lab0: pass
        return True

    def r_RV(self):
        if not self.I_pV <= self.cursor:
            return False
        return True

    def r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def r_standard_suffix(self):
        # (, line 76
        # [, line 77
        self.ket = self.cursor
        # substring, line 77
        among_var = self.find_among_b(PortugueseStemmer.a_5, 45)
        if among_var == 0:
            return False
        # ], line 77
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 92
            # call R2, line 93
            if not self.r_R2():
                return False
            # delete, line 93
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 97
            # call R2, line 98
            if not self.r_R2():
                return False
            # <-, line 98
            if not self.slice_from(u"log"):
                return False
        elif among_var == 3:
            # (, line 101
            # call R2, line 102
            if not self.r_R2():
                return False
            # <-, line 102
            if not self.slice_from(u"u"):
                return False
        elif among_var == 4:
            # (, line 105
            # call R2, line 106
            if not self.r_R2():
                return False
            # <-, line 106
            if not self.slice_from(u"ente"):
                return False
        elif among_var == 5:
            # (, line 109
            # call R1, line 110
            if not self.r_R1():
                return False
            # delete, line 110
            if not self.slice_del():
                return False

            # try, line 111
            v_1 = self.limit - self.cursor
            try:
                # (, line 111
                # [, line 112
                self.ket = self.cursor
                # substring, line 112
                among_var = self.find_among_b(PortugueseStemmer.a_2, 4)
                if among_var == 0:
                    self.cursor = self.limit - v_1
                    raise lab0()
                # ], line 112
                self.bra = self.cursor
                # call R2, line 112
                if not self.r_R2():
                    self.cursor = self.limit - v_1
                    raise lab0()
                # delete, line 112
                if not self.slice_del():
                    return False

                if among_var == 0:
                    self.cursor = self.limit - v_1
                    raise lab0()
                elif among_var == 1:
                    # (, line 113
                    # [, line 113
                    self.ket = self.cursor
                    # literal, line 113
                    if not self.eq_s_b(2, u"at"):
                        self.cursor = self.limit - v_1
                        raise lab0()
                    # ], line 113
                    self.bra = self.cursor
                    # call R2, line 113
                    if not self.r_R2():
                        self.cursor = self.limit - v_1
                        raise lab0()
                    # delete, line 113
                    if not self.slice_del():
                        return False

            except lab0: pass
        elif among_var == 6:
            # (, line 121
            # call R2, line 122
            if not self.r_R2():
                return False
            # delete, line 122
            if not self.slice_del():
                return False

            # try, line 123
            v_2 = self.limit - self.cursor
            try:
                # (, line 123
                # [, line 124
                self.ket = self.cursor
                # substring, line 124
                among_var = self.find_among_b(PortugueseStemmer.a_3, 3)
                if among_var == 0:
                    self.cursor = self.limit - v_2
                    raise lab1()
                # ], line 124
                self.bra = self.cursor
                if among_var == 0:
                    self.cursor = self.limit - v_2
                    raise lab1()
                elif among_var == 1:
                    # (, line 127
                    # call R2, line 127
                    if not self.r_R2():
                        self.cursor = self.limit - v_2
                        raise lab1()
                    # delete, line 127
                    if not self.slice_del():
                        return False

            except lab1: pass
        elif among_var == 7:
            # (, line 133
            # call R2, line 134
            if not self.r_R2():
                return False
            # delete, line 134
            if not self.slice_del():
                return False

            # try, line 135
            v_3 = self.limit - self.cursor
            try:
                # (, line 135
                # [, line 136
                self.ket = self.cursor
                # substring, line 136
                among_var = self.find_among_b(PortugueseStemmer.a_4, 3)
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab2()
                # ], line 136
                self.bra = self.cursor
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab2()
                elif among_var == 1:
                    # (, line 139
                    # call R2, line 139
                    if not self.r_R2():
                        self.cursor = self.limit - v_3
                        raise lab2()
                    # delete, line 139
                    if not self.slice_del():
                        return False

            except lab2: pass
        elif among_var == 8:
            # (, line 145
            # call R2, line 146
            if not self.r_R2():
                return False
            # delete, line 146
            if not self.slice_del():
                return False

            # try, line 147
            v_4 = self.limit - self.cursor
            try:
                # (, line 147
                # [, line 148
                self.ket = self.cursor
                # literal, line 148
                if not self.eq_s_b(2, u"at"):
                    self.cursor = self.limit - v_4
                    raise lab3()
                # ], line 148
                self.bra = self.cursor
                # call R2, line 148
                if not self.r_R2():
                    self.cursor = self.limit - v_4
                    raise lab3()
                # delete, line 148
                if not self.slice_del():
                    return False

            except lab3: pass
        elif among_var == 9:
            # (, line 152
            # call RV, line 153
            if not self.r_RV():
                return False
            # literal, line 153
            if not self.eq_s_b(1, u"e"):
                return False
            # <-, line 154
            if not self.slice_from(u"ir"):
                return False
        return True

    def r_verb_suffix(self):
        # setlimit, line 159
        v_1 = self.limit - self.cursor
        # tomark, line 159
        if self.cursor < self.I_pV:
            return False
        self.cursor = self.I_pV
        v_2 = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit - v_1
        # (, line 159
        # [, line 160
        self.ket = self.cursor
        # substring, line 160
        among_var = self.find_among_b(PortugueseStemmer.a_6, 120)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        # ], line 160
        self.bra = self.cursor
        if among_var == 0:
            self.limit_backward = v_2
            return False
        elif among_var == 1:
            # (, line 179
            # delete, line 179
            if not self.slice_del():
                return False

        self.limit_backward = v_2
        return True

    def r_residual_suffix(self):
        # (, line 183
        # [, line 184
        self.ket = self.cursor
        # substring, line 184
        among_var = self.find_among_b(PortugueseStemmer.a_7, 7)
        if among_var == 0:
            return False
        # ], line 184
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 187
            # call RV, line 187
            if not self.r_RV():
                return False
            # delete, line 187
            if not self.slice_del():
                return False

        return True

    def r_residual_form(self):
        # (, line 191
        # [, line 192
        self.ket = self.cursor
        # substring, line 192
        among_var = self.find_among_b(PortugueseStemmer.a_8, 4)
        if among_var == 0:
            return False
        # ], line 192
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 194
            # call RV, line 194
            if not self.r_RV():
                return False
            # delete, line 194
            if not self.slice_del():
                return False

            # [, line 194
            self.ket = self.cursor
            # or, line 194
            try:
                v_1 = self.limit - self.cursor
                try:
                    # (, line 194
                    # literal, line 194
                    if not self.eq_s_b(1, u"u"):
                        raise lab1()
                    # ], line 194
                    self.bra = self.cursor
                    # test, line 194
                    v_2 = self.limit - self.cursor
                    # literal, line 194
                    if not self.eq_s_b(1, u"g"):
                        raise lab1()
                    self.cursor = self.limit - v_2
                    raise lab0()
                except lab1: pass
                self.cursor = self.limit - v_1
                # (, line 195
                # literal, line 195
                if not self.eq_s_b(1, u"i"):
                    return False
                # ], line 195
                self.bra = self.cursor
                # test, line 195
                v_3 = self.limit - self.cursor
                # literal, line 195
                if not self.eq_s_b(1, u"c"):
                    return False
                self.cursor = self.limit - v_3
            except lab0: pass
            # call RV, line 195
            if not self.r_RV():
                return False
            # delete, line 195
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 196
            # <-, line 196
            if not self.slice_from(u"c"):
                return False
        return True

    def _stem(self):
        # (, line 201
        # do, line 202
        v_1 = self.cursor
        try:
            # call prelude, line 202
            if not self.r_prelude():
                raise lab0()
        except lab0: pass
        self.cursor = v_1
        # do, line 203
        v_2 = self.cursor
        try:
            # call mark_regions, line 203
            if not self.r_mark_regions():
                raise lab1()
        except lab1: pass
        self.cursor = v_2
        # backwards, line 204
        self.limit_backward = self.cursor
        self.cursor = self.limit
        # (, line 204
        # do, line 205
        v_3 = self.limit - self.cursor
        try:
            # (, line 205
            # or, line 209
            try:
                v_4 = self.limit - self.cursor
                try:
                    # (, line 206
                    # and, line 207
                    v_5 = self.limit - self.cursor
                    # (, line 206
                    # or, line 206
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            # call standard_suffix, line 206
                            if not self.r_standard_suffix():
                                raise lab6()
                            raise lab5()
                        except lab6: pass
                        self.cursor = self.limit - v_6
                        # call verb_suffix, line 206
                        if not self.r_verb_suffix():
                            raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_5
                    # do, line 207
                    v_7 = self.limit - self.cursor
                    try:
                        # (, line 207
                        # [, line 207
                        self.ket = self.cursor
                        # literal, line 207
                        if not self.eq_s_b(1, u"i"):
                            raise lab7()
                        # ], line 207
                        self.bra = self.cursor
                        # test, line 207
                        v_8 = self.limit - self.cursor
                        # literal, line 207
                        if not self.eq_s_b(1, u"c"):
                            raise lab7()
                        self.cursor = self.limit - v_8
                        # call RV, line 207
                        if not self.r_RV():
                            raise lab7()
                        # delete, line 207
                        if not self.slice_del():
                            return False

                    except lab7: pass
                    self.cursor = self.limit - v_7
                    raise lab3()
                except lab4: pass
                self.cursor = self.limit - v_4
                # call residual_suffix, line 209
                if not self.r_residual_suffix():
                    raise lab2()
            except lab3: pass
        except lab2: pass
        self.cursor = self.limit - v_3
        # do, line 211
        v_9 = self.limit - self.cursor
        try:
            # call residual_form, line 211
            if not self.r_residual_form():
                raise lab8()
        except lab8: pass
        self.cursor = self.limit - v_9
        self.cursor = self.limit_backward
        # do, line 213
        v_10 = self.cursor
        try:
            # call postlude, line 213
            if not self.r_postlude():
                raise lab9()
        except lab9: pass
        self.cursor = v_10
        return True

    def equals(self, o):
        return isinstance(o, PortugueseStemmer)

    def hashCode(self):
        return hash("PortugueseStemmer")
class lab0(BaseException): pass
class lab1(BaseException): pass
class lab2(BaseException): pass
class lab3(BaseException): pass
class lab4(BaseException): pass
class lab5(BaseException): pass
class lab6(BaseException): pass
class lab7(BaseException): pass
class lab8(BaseException): pass
class lab9(BaseException): pass
class lab10(BaseException): pass
class lab11(BaseException): pass
class lab12(BaseException): pass
class lab13(BaseException): pass
class lab14(BaseException): pass
class lab15(BaseException): pass
class lab16(BaseException): pass
class lab17(BaseException): pass
class lab18(BaseException): pass
class lab19(BaseException): pass
class lab20(BaseException): pass
class lab21(BaseException): pass



class BaseStemmer(object):
    def __init__(self):
        self.set_current("")
        self.maxCacheSize = 10000
        self._cache = {}
        self._counter = 0

    def set_current(self, value):
        '''
        Set the self.current string.
        '''
        self.current = value
        self.cursor = 0
        self.limit = len(self.current)
        self.limit_backward = 0
        self.bra = self.cursor
        self.ket = self.limit

    def get_current(self):
        '''
        Get the self.current string.
        '''
        return self.current

    def copy_from(self, other):
        self.current          = other.current
        self.cursor           = other.cursor
        self.limit            = other.limit
        self.limit_backward   = other.limit_backward
        self.bra              = other.bra
        self.ket              = other.ket

    def in_grouping(self, s, min, max):
        if self.cursor >= self.limit:
            return False
        ch = ord(self.current[self.cursor])
        if ch > max or ch < min:
            return False
        ch -= min
        if (s[ch >> 3] & (0x1 << (ch & 0x7))) == 0:
            return False
        self.cursor += 1
        return True

    def in_grouping_b(self, s, min, max):
        if self.cursor <= self.limit_backward:
            return False
        ch = ord(self.current[self.cursor - 1])
        if ch > max or ch < min:
            return False
        ch -= min
        if (s[ch >> 3] & (0x1 << (ch & 0x7))) == 0:
            return False
        self.cursor -= 1
        return True

    def out_grouping(self, s, min, max):
        if self.cursor >= self.limit:
            return False
        ch = ord(self.current[self.cursor])
        if ch > max or ch < min:
            self.cursor += 1
            return True
        ch -= min
        if (s[ch >> 3] & (0X1 << (ch & 0x7))) == 0:
            self.cursor += 1
            return True
        return False

    def out_grouping_b(self, s, min, max):
        if self.cursor <= self.limit_backward:
            return False
        ch = ord(self.current[self.cursor - 1])
        if ch > max or ch < min:
            self.cursor -= 1
            return True
        ch -= min
        if (s[ch >> 3] & (0X1 << (ch & 0x7))) == 0:
            self.cursor -= 1
            return True
        return False

    def in_range(self, min, max):
        if self.cursor >= self.limit:
            return False
        ch = ord(self.current[self.cursor])
        if ch > max or ch < min:
            return False
        self.cursor += 1
        return True

    def in_range_b(self, min, max):
        if self.cursor <= self.limit_backward:
            return False
        ch = ord(self.current[self.cursor - 1])
        if ch > max or ch < min:
            return False
        self.cursor -= 1
        return True

    def out_range(self, min, max):
        if self.cursor >= self.limit:
            return False
        ch = ord(self.current[self.cursor])
        if not (ch > max or ch < min):
            return False
        self.cursor += 1
        return True

    def out_range_b(self, min, max):
        if self.cursor <= self.limit_backward:
            return False
        ch = ord(self.current[self.cursor - 1])
        if not (ch > max or ch < min):
            return False
        self.cursor -= 1
        return True

    def eq_s(self, s_size, s):
        if self.limit - self.cursor < s_size:
            return False
        if self.current[self.cursor:self.cursor + s_size] != s:
            return False
        self.cursor += s_size
        return True

    def eq_s_b(self, s_size, s):
        if self.cursor - self.limit_backward < s_size:
            return False
        if self.current[self.cursor - s_size:self.cursor] != s:
            return False
        self.cursor -= s_size
        return True

    def eq_v(self, s):
        return self.eq_s(len(s), s)

    def eq_v_b(self, s):
        return self.eq_s_b(len(s), s)

    def find_among(self, v, v_size):
        i = 0
        j = v_size

        c = self.cursor
        l = self.limit

        common_i = 0
        common_j = 0

        first_key_inspected = False

        while True:
            k = i + ((j - i) >> 1)
            diff = 0
            common = min(common_i, common_j) # smalle
            w = v[k]
            for i2 in range(common, w.s_size):
                if c + common == l:
                    diff = -1
                    break
                diff = ord(self.current[c + common]) - ord(w.s[i2])
                if diff != 0:
                    break
                common += 1
            if diff < 0:
                j = k
                common_j = common
            else:
                i = k
                common_i = common
            if j - i <= 1:
                if i > 0:
                    break # v->s has been inspected
                if j == i:
                    break # only one item in v
                # - but now we need to go round once more to get
                # v->s inspected. self looks messy, but is actually
                # the optimal approach.
                if first_key_inspected:
                    break
                first_key_inspected = True
        while True:
            w = v[i]
            if common_i >= w.s_size:
                self.cursor = c + w.s_size
                if w.method is None:
                    return w.result
                method = getattr(self, w.method)
                res = method()
                self.cursor = c + w.s_size
                if res:
                    return w.result
            i = w.substring_i
            if i < 0:
                return 0
        return -1 # not reachable

    def find_among_b(self, v, v_size):
        '''
        find_among_b is for backwards processing. Same comments apply
        '''
        i = 0
        j = v_size

        c = self.cursor
        lb = self.limit_backward;

        common_i = 0
        common_j = 0

        first_key_inspected = False

        while True:
            k = i + ((j - i) >> 1)
            diff = 0
            common = min(common_i, common_j)
            w = v[k]
            for i2 in range(w.s_size - 1 - common, -1, -1):
                if c - common == lb:
                    diff = -1
                    break
                diff = ord(self.current[c - 1 - common]) - ord(w.s[i2])
                if diff != 0:
                    break
                common += 1
            if diff < 0:
                j = k
                common_j = common
            else:
                i = k
                common_i = common
            if j - i <= 1:
                if i > 0:
                    break
                if j == i:
                    break
                if first_key_inspected:
                    break
                first_key_inspected = True
        while True:
            w = v[i]
            if common_i >= w.s_size:
                self.cursor = c - w.s_size
                if w.method is None:
                    return w.result
                method = getattr(self, w.method)
                res = method()
                self.cursor = c - w.s_size
                if res:
                    return w.result
            i = w.substring_i
            if i < 0:
                return 0
        return -1 # not reachable

    def replace_s(self, c_bra, c_ket, s):
        '''
        to replace chars between c_bra and c_ket in self.current by the
        chars in s.

        @type c_bra int
        @type c_ket int
        @type s: string
        '''
        adjustment = len(s) - (c_ket - c_bra)
        self.current = self.current[0:c_bra] + s + self.current[c_ket:]
        self.limit += adjustment
        if self.cursor >= c_ket:
            self.cursor += adjustment
        elif self.cursor > c_bra:
            self.cursor = c_bra
        return adjustment

    def slice_check(self):
        if self.bra < 0 or self.bra > self.ket or self.ket > self.limit or self.limit > len(self.current):
            return False
        return True

    def slice_from(self, s):
        '''
        @type s string
        '''
        result = False
        if self.slice_check():
            self.replace_s(self.bra, self.ket, s)
            result = True
        return result

    def slice_del(self):
        return self.slice_from("")

    def insert(self, c_bra, c_ket, s):
        '''
        @type c_bra int
        @type c_ket int
        @type s: string
        '''
        adjustment = self.replace_s(c_bra, c_ket, s)
        if c_bra <= self.bra:
            self.bra += adjustment
        if c_bra <= self.ket:
            self.ket += adjustment

    def slice_to(self, s):
        '''
        Copy the slice into the supplied StringBuffer

        @type s: string
        '''
        result = ''
        if self.slice_check():
            result = self.current[self.bra:self.ket]
        return result

    def assign_to(self, s):
        '''
        @type s: string
        '''
        return self.current[0:self.limit]

    def _stem_word(self, word):
        cache = self._cache.get(word)
        if cache is None:
            self.set_current(word)
            self._stem()
            result = self.get_current()
            self._cache[word] = [result, self._counter]
        else:
            cache[1] = self._counter
            result = cache[0]
        self._counter += 1
        return result

    def _clear_cache(self):
        removecount = int(len(self._cache) - self.maxCacheSize * 8 / 10)
        oldcaches = sorted(self._cache.items(), key=lambda cache: cache[1][1])[0:removecount]
        for key, value in oldcaches:
            del self._cache[key]

    def stemWord(self, word):
        result = self._stem_word(word)
        if len(self._cache) > self.maxCacheSize:
            self._clear_cache()
        return result

    def stemWords(self, words):
        result = [self._stem_word(word) for word in words]
        if len(self._cache) > self.maxCacheSize:
            self._clear_cache()
        return result



# self file was generated automatically by the Snowball to Python interpreter

from .basestemmer import BaseStemmer
from .among import Among


class HungarianStemmer(BaseStemmer):
    '''
    self class was automatically generated by a Snowball to Python interpreter
    It implements the stemming algorithm defined by a snowball script.
    '''
    serialVersionUID = 1

    a_0 = [
        Among(u"cs", -1, -1),
        Among(u"dzs", -1, -1),
        Among(u"gy", -1, -1),
        Among(u"ly", -1, -1),
        Among(u"ny", -1, -1),
        Among(u"sz", -1, -1),
        Among(u"ty", -1, -1),
        Among(u"zs", -1, -1)
    ]

    a_1 = [
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 2)
    ]

    a_2 = [
        Among(u"bb", -1, -1),
        Among(u"cc", -1, -1),
        Among(u"dd", -1, -1),
        Among(u"ff", -1, -1),
        Among(u"gg", -1, -1),
        Among(u"jj", -1, -1),
        Among(u"kk", -1, -1),
        Among(u"ll", -1, -1),
        Among(u"mm", -1, -1),
        Among(u"nn", -1, -1),
        Among(u"pp", -1, -1),
        Among(u"rr", -1, -1),
        Among(u"ccs", -1, -1),
        Among(u"ss", -1, -1),
        Among(u"zzs", -1, -1),
        Among(u"tt", -1, -1),
        Among(u"vv", -1, -1),
        Among(u"ggy", -1, -1),
        Among(u"lly", -1, -1),
        Among(u"nny", -1, -1),
        Among(u"tty", -1, -1),
        Among(u"ssz", -1, -1),
        Among(u"zz", -1, -1)
    ]

    a_3 = [
        Among(u"al", -1, 1),
        Among(u"el", -1, 2)
    ]

    a_4 = [
        Among(u"ba", -1, -1),
        Among(u"ra", -1, -1),
        Among(u"be", -1, -1),
        Among(u"re", -1, -1),
        Among(u"ig", -1, -1),
        Among(u"nak", -1, -1),
        Among(u"nek", -1, -1),
        Among(u"val", -1, -1),
        Among(u"vel", -1, -1),
        Among(u"ul", -1, -1),
        Among(u"n\u00E1l", -1, -1),
        Among(u"n\u00E9l", -1, -1),
        Among(u"b\u00F3l", -1, -1),
        Among(u"r\u00F3l", -1, -1),
        Among(u"t\u00F3l", -1, -1),
        Among(u"b\u00F5l", -1, -1),
        Among(u"r\u00F5l", -1, -1),
        Among(u"t\u00F5l", -1, -1),
        Among(u"\u00FCl", -1, -1),
        Among(u"n", -1, -1),
        Among(u"an", 19, -1),
        Among(u"ban", 20, -1),
        Among(u"en", 19, -1),
        Among(u"ben", 22, -1),
        Among(u"k\u00E9ppen", 22, -1),
        Among(u"on", 19, -1),
        Among(u"\u00F6n", 19, -1),
        Among(u"k\u00E9pp", -1, -1),
        Among(u"kor", -1, -1),
        Among(u"t", -1, -1),
        Among(u"at", 29, -1),
        Among(u"et", 29, -1),
        Among(u"k\u00E9nt", 29, -1),
        Among(u"ank\u00E9nt", 32, -1),
        Among(u"enk\u00E9nt", 32, -1),
        Among(u"onk\u00E9nt", 32, -1),
        Among(u"ot", 29, -1),
        Among(u"\u00E9rt", 29, -1),
        Among(u"\u00F6t", 29, -1),
        Among(u"hez", -1, -1),
        Among(u"hoz", -1, -1),
        Among(u"h\u00F6z", -1, -1),
        Among(u"v\u00E1", -1, -1),
        Among(u"v\u00E9", -1, -1)
    ]

    a_5 = [
        Among(u"\u00E1n", -1, 2),
        Among(u"\u00E9n", -1, 1),
        Among(u"\u00E1nk\u00E9nt", -1, 3)
    ]

    a_6 = [
        Among(u"stul", -1, 2),
        Among(u"astul", 0, 1),
        Among(u"\u00E1stul", 0, 3),
        Among(u"st\u00FCl", -1, 2),
        Among(u"est\u00FCl", 3, 1),
        Among(u"\u00E9st\u00FCl", 3, 4)
    ]

    a_7 = [
        Among(u"\u00E1", -1, 1),
        Among(u"\u00E9", -1, 2)
    ]

    a_8 = [
        Among(u"k", -1, 7),
        Among(u"ak", 0, 4),
        Among(u"ek", 0, 6),
        Among(u"ok", 0, 5),
        Among(u"\u00E1k", 0, 1),
        Among(u"\u00E9k", 0, 2),
        Among(u"\u00F6k", 0, 3)
    ]

    a_9 = [
        Among(u"\u00E9i", -1, 7),
        Among(u"\u00E1\u00E9i", 0, 6),
        Among(u"\u00E9\u00E9i", 0, 5),
        Among(u"\u00E9", -1, 9),
        Among(u"k\u00E9", 3, 4),
        Among(u"ak\u00E9", 4, 1),
        Among(u"ek\u00E9", 4, 1),
        Among(u"ok\u00E9", 4, 1),
        Among(u"\u00E1k\u00E9", 4, 3),
        Among(u"\u00E9k\u00E9", 4, 2),
        Among(u"\u00F6k\u00E9", 4, 1),
        Among(u"\u00E9\u00E9", 3, 8)
    ]

    a_10 = [
        Among(u"a", -1, 18),
        Among(u"ja", 0, 17),
        Among(u"d", -1, 16),
        Among(u"ad", 2, 13),
        Among(u"ed", 2, 13),
        Among(u"od", 2, 13),
        Among(u"\u00E1d", 2, 14),
        Among(u"\u00E9d", 2, 15),
        Among(u"\u00F6d", 2, 13),
        Among(u"e", -1, 18),
        Among(u"je", 9, 17),
        Among(u"nk", -1, 4),
        Among(u"unk", 11, 1),
        Among(u"\u00E1nk", 11, 2),
        Among(u"\u00E9nk", 11, 3),
        Among(u"\u00FCnk", 11, 1),
        Among(u"uk", -1, 8),
        Among(u"juk", 16, 7),
        Among(u"\u00E1juk", 17, 5),
        Among(u"\u00FCk", -1, 8),
        Among(u"j\u00FCk", 19, 7),
        Among(u"\u00E9j\u00FCk", 20, 6),
        Among(u"m", -1, 12),
        Among(u"am", 22, 9),
        Among(u"em", 22, 9),
        Among(u"om", 22, 9),
        Among(u"\u00E1m", 22, 10),
        Among(u"\u00E9m", 22, 11),
        Among(u"o", -1, 18),
        Among(u"\u00E1", -1, 19),
        Among(u"\u00E9", -1, 20)
    ]

    a_11 = [
        Among(u"id", -1, 10),
        Among(u"aid", 0, 9),
        Among(u"jaid", 1, 6),
        Among(u"eid", 0, 9),
        Among(u"jeid", 3, 6),
        Among(u"\u00E1id", 0, 7),
        Among(u"\u00E9id", 0, 8),
        Among(u"i", -1, 15),
        Among(u"ai", 7, 14),
        Among(u"jai", 8, 11),
        Among(u"ei", 7, 14),
        Among(u"jei", 10, 11),
        Among(u"\u00E1i", 7, 12),
        Among(u"\u00E9i", 7, 13),
        Among(u"itek", -1, 24),
        Among(u"eitek", 14, 21),
        Among(u"jeitek", 15, 20),
        Among(u"\u00E9itek", 14, 23),
        Among(u"ik", -1, 29),
        Among(u"aik", 18, 26),
        Among(u"jaik", 19, 25),
        Among(u"eik", 18, 26),
        Among(u"jeik", 21, 25),
        Among(u"\u00E1ik", 18, 27),
        Among(u"\u00E9ik", 18, 28),
        Among(u"ink", -1, 20),
        Among(u"aink", 25, 17),
        Among(u"jaink", 26, 16),
        Among(u"eink", 25, 17),
        Among(u"jeink", 28, 16),
        Among(u"\u00E1ink", 25, 18),
        Among(u"\u00E9ink", 25, 19),
        Among(u"aitok", -1, 21),
        Among(u"jaitok", 32, 20),
        Among(u"\u00E1itok", -1, 22),
        Among(u"im", -1, 5),
        Among(u"aim", 35, 4),
        Among(u"jaim", 36, 1),
        Among(u"eim", 35, 4),
        Among(u"jeim", 38, 1),
        Among(u"\u00E1im", 35, 2),
        Among(u"\u00E9im", 35, 3)
    ]

    g_v = [17, 65, 16, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 17, 52, 14]

    I_p1 = 0

    def copy_from(self, other):
        self.I_p1 = other.I_p1
        super.copy_from(other)
    

    def r_mark_regions(self):
        # (, line 44
        self.I_p1 = self.limit;
        # or, line 51
        try:
            v_1 = self.cursor
            try:
                # (, line 48
                if not self.in_grouping(HungarianStemmer.g_v, 97, 252):
                    raise lab1()
                # goto, line 48
                try:
                    while True:
                        v_2 = self.cursor
                        try:
                            if not self.out_grouping(HungarianStemmer.g_v, 97, 252):
                                raise lab3()
                            self.cursor = v_2
                            raise lab2()
                        except lab3: pass
                        self.cursor = v_2
                        if self.cursor >= self.limit:
                            raise lab1()
                        self.cursor += 1
                except lab2: pass
                # or, line 49
                try:
                    v_3 = self.cursor
                    try:
                        # among, line 49
                        if self.find_among(HungarianStemmer.a_0, 8) == 0:
                            raise lab5()
                        raise lab4()
                    except lab5: pass
                    self.cursor = v_3
                    # next, line 49
                    if self.cursor >= self.limit:
                        raise lab1()
                    self.cursor += 1
                except lab4: pass
                # setmark p1, line 50
                self.I_p1 = self.cursor
                raise lab0()
            except lab1: pass
            self.cursor = v_1
            # (, line 53
            if not self.out_grouping(HungarianStemmer.g_v, 97, 252):
                return False
            # gopast, line 53
            try:
                while True:
                    try:
                        if not self.in_grouping(HungarianStemmer.g_v, 97, 252):
                            raise lab7()
                        raise lab6()
                    except lab7: pass
                    if self.cursor >= self.limit:
                        return False
                    self.cursor += 1
            except lab6: pass
            # setmark p1, line 53
            self.I_p1 = self.cursor
        except lab0: pass
        return True

    def r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def r_v_ending(self):
        # (, line 60
        # [, line 61
        self.ket = self.cursor
        # substring, line 61
        among_var = self.find_among_b(HungarianStemmer.a_1, 2)
        if among_var == 0:
            return False
        # ], line 61
        self.bra = self.cursor
        # call R1, line 61
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 62
            # <-, line 62
            if not self.slice_from(u"a"):
                return False
        elif among_var == 2:
            # (, line 63
            # <-, line 63
            if not self.slice_from(u"e"):
                return False
        return True

    def r_double(self):
        # (, line 67
        # test, line 68
        v_1 = self.limit - self.cursor
        # among, line 68
        if self.find_among_b(HungarianStemmer.a_2, 23) == 0:
            return False
        self.cursor = self.limit - v_1
        return True

    def r_undouble(self):
        # (, line 72
        # next, line 73
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        # [, line 73
        self.ket = self.cursor
        # hop, line 73
        c = self.cursor - 1
        if self.limit_backward > c or c > self.limit:
            return False
        self.cursor = c
        # ], line 73
        self.bra = self.cursor
        # delete, line 73
        if not self.slice_del():
            return False

        return True

    def r_instrum(self):
        # (, line 76
        # [, line 77
        self.ket = self.cursor
        # substring, line 77
        among_var = self.find_among_b(HungarianStemmer.a_3, 2)
        if among_var == 0:
            return False
        # ], line 77
        self.bra = self.cursor
        # call R1, line 77
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 78
            # call double, line 78
            if not self.r_double():
                return False
        elif among_var == 2:
            # (, line 79
            # call double, line 79
            if not self.r_double():
                return False
        # delete, line 81
        if not self.slice_del():
            return False

        # call undouble, line 82
        if not self.r_undouble():
            return False
        return True

    def r_case(self):
        # (, line 86
        # [, line 87
        self.ket = self.cursor
        # substring, line 87
        if self.find_among_b(HungarianStemmer.a_4, 44) == 0:
            return False
        # ], line 87
        self.bra = self.cursor
        # call R1, line 87
        if not self.r_R1():
            return False
        # delete, line 111
        if not self.slice_del():
            return False

        # call v_ending, line 112
        if not self.r_v_ending():
            return False
        return True

    def r_case_special(self):
        # (, line 115
        # [, line 116
        self.ket = self.cursor
        # substring, line 116
        among_var = self.find_among_b(HungarianStemmer.a_5, 3)
        if among_var == 0:
            return False
        # ], line 116
        self.bra = self.cursor
        # call R1, line 116
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 117
            # <-, line 117
            if not self.slice_from(u"e"):
                return False
        elif among_var == 2:
            # (, line 118
            # <-, line 118
            if not self.slice_from(u"a"):
                return False
        elif among_var == 3:
            # (, line 119
            # <-, line 119
            if not self.slice_from(u"a"):
                return False
        return True

    def r_case_other(self):
        # (, line 123
        # [, line 124
        self.ket = self.cursor
        # substring, line 124
        among_var = self.find_among_b(HungarianStemmer.a_6, 6)
        if among_var == 0:
            return False
        # ], line 124
        self.bra = self.cursor
        # call R1, line 124
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 125
            # delete, line 125
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 126
            # delete, line 126
            if not self.slice_del():
                return False

        elif among_var == 3:
            # (, line 127
            # <-, line 127
            if not self.slice_from(u"a"):
                return False
        elif among_var == 4:
            # (, line 128
            # <-, line 128
            if not self.slice_from(u"e"):
                return False
        return True

    def r_factive(self):
        # (, line 132
        # [, line 133
        self.ket = self.cursor
        # substring, line 133
        among_var = self.find_among_b(HungarianStemmer.a_7, 2)
        if among_var == 0:
            return False
        # ], line 133
        self.bra = self.cursor
        # call R1, line 133
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 134
            # call double, line 134
            if not self.r_double():
                return False
        elif among_var == 2:
            # (, line 135
            # call double, line 135
            if not self.r_double():
                return False
        # delete, line 137
        if not self.slice_del():
            return False

        # call undouble, line 138
        if not self.r_undouble():
            return False
        return True

    def r_plural(self):
        # (, line 141
        # [, line 142
        self.ket = self.cursor
        # substring, line 142
        among_var = self.find_among_b(HungarianStemmer.a_8, 7)
        if among_var == 0:
            return False
        # ], line 142
        self.bra = self.cursor
        # call R1, line 142
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 143
            # <-, line 143
            if not self.slice_from(u"a"):
                return False
        elif among_var == 2:
            # (, line 144
            # <-, line 144
            if not self.slice_from(u"e"):
                return False
        elif among_var == 3:
            # (, line 145
            # delete, line 145
            if not self.slice_del():
                return False

        elif among_var == 4:
            # (, line 146
            # delete, line 146
            if not self.slice_del():
                return False

        elif among_var == 5:
            # (, line 147
            # delete, line 147
            if not self.slice_del():
                return False

        elif among_var == 6:
            # (, line 148
            # delete, line 148
            if not self.slice_del():
                return False

        elif among_var == 7:
            # (, line 149
            # delete, line 149
            if not self.slice_del():
                return False

        return True

    def r_owned(self):
        # (, line 153
        # [, line 154
        self.ket = self.cursor
        # substring, line 154
        among_var = self.find_among_b(HungarianStemmer.a_9, 12)
        if among_var == 0:
            return False
        # ], line 154
        self.bra = self.cursor
        # call R1, line 154
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 155
            # delete, line 155
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 156
            # <-, line 156
            if not self.slice_from(u"e"):
                return False
        elif among_var == 3:
            # (, line 157
            # <-, line 157
            if not self.slice_from(u"a"):
                return False
        elif among_var == 4:
            # (, line 158
            # delete, line 158
            if not self.slice_del():
                return False

        elif among_var == 5:
            # (, line 159
            # <-, line 159
            if not self.slice_from(u"e"):
                return False
        elif among_var == 6:
            # (, line 160
            # <-, line 160
            if not self.slice_from(u"a"):
                return False
        elif among_var == 7:
            # (, line 161
            # delete, line 161
            if not self.slice_del():
                return False

        elif among_var == 8:
            # (, line 162
            # <-, line 162
            if not self.slice_from(u"e"):
                return False
        elif among_var == 9:
            # (, line 163
            # delete, line 163
            if not self.slice_del():
                return False

        return True

    def r_sing_owner(self):
        # (, line 167
        # [, line 168
        self.ket = self.cursor
        # substring, line 168
        among_var = self.find_among_b(HungarianStemmer.a_10, 31)
        if among_var == 0:
            return False
        # ], line 168
        self.bra = self.cursor
        # call R1, line 168
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 169
            # delete, line 169
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 170
            # <-, line 170
            if not self.slice_from(u"a"):
                return False
        elif among_var == 3:
            # (, line 171
            # <-, line 171
            if not self.slice_from(u"e"):
                return False
        elif among_var == 4:
            # (, line 172
            # delete, line 172
            if not self.slice_del():
                return False

        elif among_var == 5:
            # (, line 173
            # <-, line 173
            if not self.slice_from(u"a"):
                return False
        elif among_var == 6:
            # (, line 174
            # <-, line 174
            if not self.slice_from(u"e"):
                return False
        elif among_var == 7:
            # (, line 175
            # delete, line 175
            if not self.slice_del():
                return False

        elif among_var == 8:
            # (, line 176
            # delete, line 176
            if not self.slice_del():
                return False

        elif among_var == 9:
            # (, line 177
            # delete, line 177
            if not self.slice_del():
                return False

        elif among_var == 10:
            # (, line 178
            # <-, line 178
            if not self.slice_from(u"a"):
                return False
        elif among_var == 11:
            # (, line 179
            # <-, line 179
            if not self.slice_from(u"e"):
                return False
        elif among_var == 12:
            # (, line 180
            # delete, line 180
            if not self.slice_del():
                return False

        elif among_var == 13:
            # (, line 181
            # delete, line 181
            if not self.slice_del():
                return False

        elif among_var == 14:
            # (, line 182
            # <-, line 182
            if not self.slice_from(u"a"):
                return False
        elif among_var == 15:
            # (, line 183
            # <-, line 183
            if not self.slice_from(u"e"):
                return False
        elif among_var == 16:
            # (, line 184
            # delete, line 184
            if not self.slice_del():
                return False

        elif among_var == 17:
            # (, line 185
            # delete, line 185
            if not self.slice_del():
                return False

        elif among_var == 18:
            # (, line 186
            # delete, line 186
            if not self.slice_del():
                return False

        elif among_var == 19:
            # (, line 187
            # <-, line 187
            if not self.slice_from(u"a"):
                return False
        elif among_var == 20:
            # (, line 188
            # <-, line 188
            if not self.slice_from(u"e"):
                return False
        return True

    def r_plur_owner(self):
        # (, line 192
        # [, line 193
        self.ket = self.cursor
        # substring, line 193
        among_var = self.find_among_b(HungarianStemmer.a_11, 42)
        if among_var == 0:
            return False
        # ], line 193
        self.bra = self.cursor
        # call R1, line 193
        if not self.r_R1():
            return False
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 194
            # delete, line 194
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 195
            # <-, line 195
            if not self.slice_from(u"a"):
                return False
        elif among_var == 3:
            # (, line 196
            # <-, line 196
            if not self.slice_from(u"e"):
                return False
        elif among_var == 4:
            # (, line 197
            # delete, line 197
            if not self.slice_del():
                return False

        elif among_var == 5:
            # (, line 198
            # delete, line 198
            if not self.slice_del():
                return False

        elif among_var == 6:
            # (, line 199
            # delete, line 199
            if not self.slice_del():
                return False

        elif among_var == 7:
            # (, line 200
            # <-, line 200
            if not self.slice_from(u"a"):
                return False
        elif among_var == 8:
            # (, line 201
            # <-, line 201
            if not self.slice_from(u"e"):
                return False
        elif among_var == 9:
            # (, line 202
            # delete, line 202
            if not self.slice_del():
                return False

        elif among_var == 10:
            # (, line 203
            # delete, line 203
            if not self.slice_del():
                return False

        elif among_var == 11:
            # (, line 204
            # delete, line 204
            if not self.slice_del():
                return False

        elif among_var == 12:
            # (, line 205
            # <-, line 205
            if not self.slice_from(u"a"):
                return False
        elif among_var == 13:
            # (, line 206
            # <-, line 206
            if not self.slice_from(u"e"):
                return False
        elif among_var == 14:
            # (, line 207
            # delete, line 207
            if not self.slice_del():
                return False

        elif among_var == 15:
            # (, line 208
            # delete, line 208
            if not self.slice_del():
                return False

        elif among_var == 16:
            # (, line 209
            # delete, line 209
            if not self.slice_del():
                return False

        elif among_var == 17:
            # (, line 210
            # delete, line 210
            if not self.slice_del():
                return False

        elif among_var == 18:
            # (, line 211
            # <-, line 211
            if not self.slice_from(u"a"):
                return False
        elif among_var == 19:
            # (, line 212
            # <-, line 212
            if not self.slice_from(u"e"):
                return False
        elif among_var == 20:
            # (, line 214
            # delete, line 214
            if not self.slice_del():
                return False

        elif among_var == 21:
            # (, line 215
            # delete, line 215
            if not self.slice_del():
                return False

        elif among_var == 22:
            # (, line 216
            # <-, line 216
            if not self.slice_from(u"a"):
                return False
        elif among_var == 23:
            # (, line 217
            # <-, line 217
            if not self.slice_from(u"e"):
                return False
        elif among_var == 24:
            # (, line 218
            # delete, line 218
            if not self.slice_del():
                return False

        elif among_var == 25:
            # (, line 219
            # delete, line 219
            if not self.slice_del():
                return False

        elif among_var == 26:
            # (, line 220
            # delete, line 220
            if not self.slice_del():
                return False

        elif among_var == 27:
            # (, line 221
            # <-, line 221
            if not self.slice_from(u"a"):
                return False
        elif among_var == 28:
            # (, line 222
            # <-, line 222
            if not self.slice_from(u"e"):
                return False
        elif among_var == 29:
            # (, line 223
            # delete, line 223
            if not self.slice_del():
                return False

        return True

    def _stem(self):
        # (, line 228
        # do, line 229
        v_1 = self.cursor
        try:
            # call mark_regions, line 229
            if not self.r_mark_regions():
                raise lab0()
        except lab0: pass
        self.cursor = v_1
        # backwards, line 230
        self.limit_backward = self.cursor
        self.cursor = self.limit
        # (, line 230
        # do, line 231
        v_2 = self.limit - self.cursor
        try:
            # call instrum, line 231
            if not self.r_instrum():
                raise lab1()
        except lab1: pass
        self.cursor = self.limit - v_2
        # do, line 232
        v_3 = self.limit - self.cursor
        try:
            # call case, line 232
            if not self.r_case():
                raise lab2()
        except lab2: pass
        self.cursor = self.limit - v_3
        # do, line 233
        v_4 = self.limit - self.cursor
        try:
            # call case_special, line 233
            if not self.r_case_special():
                raise lab3()
        except lab3: pass
        self.cursor = self.limit - v_4
        # do, line 234
        v_5 = self.limit - self.cursor
        try:
            # call case_other, line 234
            if not self.r_case_other():
                raise lab4()
        except lab4: pass
        self.cursor = self.limit - v_5
        # do, line 235
        v_6 = self.limit - self.cursor
        try:
            # call factive, line 235
            if not self.r_factive():
                raise lab5()
        except lab5: pass
        self.cursor = self.limit - v_6
        # do, line 236
        v_7 = self.limit - self.cursor
        try:
            # call owned, line 236
            if not self.r_owned():
                raise lab6()
        except lab6: pass
        self.cursor = self.limit - v_7
        # do, line 237
        v_8 = self.limit - self.cursor
        try:
            # call sing_owner, line 237
            if not self.r_sing_owner():
                raise lab7()
        except lab7: pass
        self.cursor = self.limit - v_8
        # do, line 238
        v_9 = self.limit - self.cursor
        try:
            # call plur_owner, line 238
            if not self.r_plur_owner():
                raise lab8()
        except lab8: pass
        self.cursor = self.limit - v_9
        # do, line 239
        v_10 = self.limit - self.cursor
        try:
            # call plural, line 239
            if not self.r_plural():
                raise lab9()
        except lab9: pass
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        return True

    def equals(self, o):
        return isinstance(o, HungarianStemmer)

    def hashCode(self):
        return hash("HungarianStemmer")
class lab0(BaseException): pass
class lab1(BaseException): pass
class lab2(BaseException): pass
class lab3(BaseException): pass
class lab4(BaseException): pass
class lab5(BaseException): pass
class lab6(BaseException): pass
class lab7(BaseException): pass
class lab8(BaseException): pass
class lab9(BaseException): pass



# self file was generated automatically by the Snowball to Python interpreter

from .basestemmer import BaseStemmer
from .among import Among


class FrenchStemmer(BaseStemmer):
    '''
    self class was automatically generated by a Snowball to Python interpreter
    It implements the stemming algorithm defined by a snowball script.
    '''
    serialVersionUID = 1

    a_0 = [
        Among(u"col", -1, -1),
        Among(u"par", -1, -1),
        Among(u"tap", -1, -1)
    ]

    a_1 = [
        Among(u"", -1, 4),
        Among(u"I", 0, 1),
        Among(u"U", 0, 2),
        Among(u"Y", 0, 3)
    ]

    a_2 = [
        Among(u"iqU", -1, 3),
        Among(u"abl", -1, 3),
        Among(u"I\u00E8r", -1, 4),
        Among(u"i\u00E8r", -1, 4),
        Among(u"eus", -1, 2),
        Among(u"iv", -1, 1)
    ]

    a_3 = [
        Among(u"ic", -1, 2),
        Among(u"abil", -1, 1),
        Among(u"iv", -1, 3)
    ]

    a_4 = [
        Among(u"iqUe", -1, 1),
        Among(u"atrice", -1, 2),
        Among(u"ance", -1, 1),
        Among(u"ence", -1, 5),
        Among(u"logie", -1, 3),
        Among(u"able", -1, 1),
        Among(u"isme", -1, 1),
        Among(u"euse", -1, 11),
        Among(u"iste", -1, 1),
        Among(u"ive", -1, 8),
        Among(u"if", -1, 8),
        Among(u"usion", -1, 4),
        Among(u"ation", -1, 2),
        Among(u"ution", -1, 4),
        Among(u"ateur", -1, 2),
        Among(u"iqUes", -1, 1),
        Among(u"atrices", -1, 2),
        Among(u"ances", -1, 1),
        Among(u"ences", -1, 5),
        Among(u"logies", -1, 3),
        Among(u"ables", -1, 1),
        Among(u"ismes", -1, 1),
        Among(u"euses", -1, 11),
        Among(u"istes", -1, 1),
        Among(u"ives", -1, 8),
        Among(u"ifs", -1, 8),
        Among(u"usions", -1, 4),
        Among(u"ations", -1, 2),
        Among(u"utions", -1, 4),
        Among(u"ateurs", -1, 2),
        Among(u"ments", -1, 15),
        Among(u"ements", 30, 6),
        Among(u"issements", 31, 12),
        Among(u"it\u00E9s", -1, 7),
        Among(u"ment", -1, 15),
        Among(u"ement", 34, 6),
        Among(u"issement", 35, 12),
        Among(u"amment", 34, 13),
        Among(u"emment", 34, 14),
        Among(u"aux", -1, 10),
        Among(u"eaux", 39, 9),
        Among(u"eux", -1, 1),
        Among(u"it\u00E9", -1, 7)
    ]

    a_5 = [
        Among(u"ira", -1, 1),
        Among(u"ie", -1, 1),
        Among(u"isse", -1, 1),
        Among(u"issante", -1, 1),
        Among(u"i", -1, 1),
        Among(u"irai", 4, 1),
        Among(u"ir", -1, 1),
        Among(u"iras", -1, 1),
        Among(u"ies", -1, 1),
        Among(u"\u00EEmes", -1, 1),
        Among(u"isses", -1, 1),
        Among(u"issantes", -1, 1),
        Among(u"\u00EEtes", -1, 1),
        Among(u"is", -1, 1),
        Among(u"irais", 13, 1),
        Among(u"issais", 13, 1),
        Among(u"irions", -1, 1),
        Among(u"issions", -1, 1),
        Among(u"irons", -1, 1),
        Among(u"issons", -1, 1),
        Among(u"issants", -1, 1),
        Among(u"it", -1, 1),
        Among(u"irait", 21, 1),
        Among(u"issait", 21, 1),
        Among(u"issant", -1, 1),
        Among(u"iraIent", -1, 1),
        Among(u"issaIent", -1, 1),
        Among(u"irent", -1, 1),
        Among(u"issent", -1, 1),
        Among(u"iront", -1, 1),
        Among(u"\u00EEt", -1, 1),
        Among(u"iriez", -1, 1),
        Among(u"issiez", -1, 1),
        Among(u"irez", -1, 1),
        Among(u"issez", -1, 1)
    ]

    a_6 = [
        Among(u"a", -1, 3),
        Among(u"era", 0, 2),
        Among(u"asse", -1, 3),
        Among(u"ante", -1, 3),
        Among(u"\u00E9e", -1, 2),
        Among(u"ai", -1, 3),
        Among(u"erai", 5, 2),
        Among(u"er", -1, 2),
        Among(u"as", -1, 3),
        Among(u"eras", 8, 2),
        Among(u"\u00E2mes", -1, 3),
        Among(u"asses", -1, 3),
        Among(u"antes", -1, 3),
        Among(u"\u00E2tes", -1, 3),
        Among(u"\u00E9es", -1, 2),
        Among(u"ais", -1, 3),
        Among(u"erais", 15, 2),
        Among(u"ions", -1, 1),
        Among(u"erions", 17, 2),
        Among(u"assions", 17, 3),
        Among(u"erons", -1, 2),
        Among(u"ants", -1, 3),
        Among(u"\u00E9s", -1, 2),
        Among(u"ait", -1, 3),
        Among(u"erait", 23, 2),
        Among(u"ant", -1, 3),
        Among(u"aIent", -1, 3),
        Among(u"eraIent", 26, 2),
        Among(u"\u00E8rent", -1, 2),
        Among(u"assent", -1, 3),
        Among(u"eront", -1, 2),
        Among(u"\u00E2t", -1, 3),
        Among(u"ez", -1, 2),
        Among(u"iez", 32, 2),
        Among(u"eriez", 33, 2),
        Among(u"assiez", 33, 3),
        Among(u"erez", 32, 2),
        Among(u"\u00E9", -1, 2)
    ]

    a_7 = [
        Among(u"e", -1, 3),
        Among(u"I\u00E8re", 0, 2),
        Among(u"i\u00E8re", 0, 2),
        Among(u"ion", -1, 1),
        Among(u"Ier", -1, 2),
        Among(u"ier", -1, 2),
        Among(u"\u00EB", -1, 4)
    ]

    a_8 = [
        Among(u"ell", -1, -1),
        Among(u"eill", -1, -1),
        Among(u"enn", -1, -1),
        Among(u"onn", -1, -1),
        Among(u"ett", -1, -1)
    ]

    g_v = [17, 65, 16, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128, 130, 103, 8, 5]

    g_keep_with_s = [1, 65, 20, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 128]

    I_p2 = 0
    I_p1 = 0
    I_pV = 0

    def copy_from(self, other):
        self.I_p2 = other.I_p2
        self.I_p1 = other.I_p1
        self.I_pV = other.I_pV
        super.copy_from(other)
    

    def r_prelude(self):
        # repeat, line 38
        try:
            while True:
                try:
                    v_1 = self.cursor
                    try:
                        # goto, line 38
                        try:
                            while True:
                                v_2 = self.cursor
                                try:
                                    # (, line 38
                                    # or, line 44
                                    try:
                                        v_3 = self.cursor
                                        try:
                                            # (, line 40
                                            if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                raise lab6()
                                            # [, line 40
                                            self.bra = self.cursor
                                            # or, line 40
                                            try:
                                                v_4 = self.cursor
                                                try:
                                                    # (, line 40
                                                    # literal, line 40
                                                    if not self.eq_s(1, u"u"):
                                                        raise lab8()
                                                    # ], line 40
                                                    self.ket = self.cursor
                                                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                        raise lab8()
                                                    # <-, line 40
                                                    if not self.slice_from(u"U"):
                                                        return False
                                                    raise lab7()
                                                except lab8: pass
                                                self.cursor = v_4
                                                try:
                                                    # (, line 41
                                                    # literal, line 41
                                                    if not self.eq_s(1, u"i"):
                                                        raise lab9()
                                                    # ], line 41
                                                    self.ket = self.cursor
                                                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                        raise lab9()
                                                    # <-, line 41
                                                    if not self.slice_from(u"I"):
                                                        return False
                                                    raise lab7()
                                                except lab9: pass
                                                self.cursor = v_4
                                                # (, line 42
                                                # literal, line 42
                                                if not self.eq_s(1, u"y"):
                                                    raise lab6()
                                                # ], line 42
                                                self.ket = self.cursor
                                                # <-, line 42
                                                if not self.slice_from(u"Y"):
                                                    return False
                                            except lab7: pass
                                            raise lab5()
                                        except lab6: pass
                                        self.cursor = v_3
                                        try:
                                            # (, line 45
                                            # [, line 45
                                            self.bra = self.cursor
                                            # literal, line 45
                                            if not self.eq_s(1, u"y"):
                                                raise lab10()
                                            # ], line 45
                                            self.ket = self.cursor
                                            if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                                raise lab10()
                                            # <-, line 45
                                            if not self.slice_from(u"Y"):
                                                return False
                                            raise lab5()
                                        except lab10: pass
                                        self.cursor = v_3
                                        # (, line 47
                                        # literal, line 47
                                        if not self.eq_s(1, u"q"):
                                            raise lab4()
                                        # [, line 47
                                        self.bra = self.cursor
                                        # literal, line 47
                                        if not self.eq_s(1, u"u"):
                                            raise lab4()
                                        # ], line 47
                                        self.ket = self.cursor
                                        # <-, line 47
                                        if not self.slice_from(u"U"):
                                            return False
                                    except lab5: pass
                                    self.cursor = v_2
                                    raise lab3()
                                except lab4: pass
                                self.cursor = v_2
                                if self.cursor >= self.limit:
                                    raise lab2()
                                self.cursor += 1
                        except lab3: pass
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_1
                    raise lab0()
                except lab1: pass
        except lab0: pass
        return True

    def r_mark_regions(self):
        # (, line 50
        self.I_pV = self.limit;
        self.I_p1 = self.limit;
        self.I_p2 = self.limit;
        # do, line 56
        v_1 = self.cursor
        try:
            # (, line 56
            # or, line 58
            try:
                v_2 = self.cursor
                try:
                    # (, line 57
                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                        raise lab2()
                    if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                        raise lab2()
                    # next, line 57
                    if self.cursor >= self.limit:
                        raise lab2()
                    self.cursor += 1
                    raise lab1()
                except lab2: pass
                self.cursor = v_2
                try:
                    # among, line 59
                    if self.find_among(FrenchStemmer.a_0, 3) == 0:
                        raise lab3()
                    raise lab1()
                except lab3: pass
                self.cursor = v_2
                # (, line 66
                # next, line 66
                if self.cursor >= self.limit:
                    raise lab0()
                self.cursor += 1
                # gopast, line 66
                try:
                    while True:
                        try:
                            if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                                raise lab5()
                            raise lab4()
                        except lab5: pass
                        if self.cursor >= self.limit:
                            raise lab0()
                        self.cursor += 1
                except lab4: pass
            except lab1: pass
            # setmark pV, line 67
            self.I_pV = self.cursor
        except lab0: pass
        self.cursor = v_1
        # do, line 69
        v_4 = self.cursor
        try:
            # (, line 69
            # gopast, line 70
            try:
                while True:
                    try:
                        if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                            raise lab8()
                        raise lab7()
                    except lab8: pass
                    if self.cursor >= self.limit:
                        raise lab6()
                    self.cursor += 1
            except lab7: pass
            # gopast, line 70
            try:
                while True:
                    try:
                        if not self.out_grouping(FrenchStemmer.g_v, 97, 251):
                            raise lab10()
                        raise lab9()
                    except lab10: pass
                    if self.cursor >= self.limit:
                        raise lab6()
                    self.cursor += 1
            except lab9: pass
            # setmark p1, line 70
            self.I_p1 = self.cursor
            # gopast, line 71
            try:
                while True:
                    try:
                        if not self.in_grouping(FrenchStemmer.g_v, 97, 251):
                            raise lab12()
                        raise lab11()
                    except lab12: pass
                    if self.cursor >= self.limit:
                        raise lab6()
                    self.cursor += 1
            except lab11: pass
            # gopast, line 71
            try:
                while True:
                    try:
                        if not self.out_grouping(FrenchStemmer.g_v, 97, 251):
                            raise lab14()
                        raise lab13()
                    except lab14: pass
                    if self.cursor >= self.limit:
                        raise lab6()
                    self.cursor += 1
            except lab13: pass
            # setmark p2, line 71
            self.I_p2 = self.cursor
        except lab6: pass
        self.cursor = v_4
        return True

    def r_postlude(self):
        # repeat, line 75
        try:
            while True:
                try:
                    v_1 = self.cursor
                    try:
                        # (, line 75
                        # [, line 77
                        self.bra = self.cursor
                        # substring, line 77
                        among_var = self.find_among(FrenchStemmer.a_1, 4)
                        if among_var == 0:
                            raise lab2()
                        # ], line 77
                        self.ket = self.cursor
                        if among_var == 0:
                            raise lab2()
                        elif among_var == 1:
                            # (, line 78
                            # <-, line 78
                            if not self.slice_from(u"i"):
                                return False
                        elif among_var == 2:
                            # (, line 79
                            # <-, line 79
                            if not self.slice_from(u"u"):
                                return False
                        elif among_var == 3:
                            # (, line 80
                            # <-, line 80
                            if not self.slice_from(u"y"):
                                return False
                        elif among_var == 4:
                            # (, line 81
                            # next, line 81
                            if self.cursor >= self.limit:
                                raise lab2()
                            self.cursor += 1
                        raise lab1()
                    except lab2: pass
                    self.cursor = v_1
                    raise lab0()
                except lab1: pass
        except lab0: pass
        return True

    def r_RV(self):
        if not self.I_pV <= self.cursor:
            return False
        return True

    def r_R1(self):
        if not self.I_p1 <= self.cursor:
            return False
        return True

    def r_R2(self):
        if not self.I_p2 <= self.cursor:
            return False
        return True

    def r_standard_suffix(self):
        # (, line 91
        # [, line 92
        self.ket = self.cursor
        # substring, line 92
        among_var = self.find_among_b(FrenchStemmer.a_4, 43)
        if among_var == 0:
            return False
        # ], line 92
        self.bra = self.cursor
        if among_var == 0:
            return False
        elif among_var == 1:
            # (, line 96
            # call R2, line 96
            if not self.r_R2():
                return False
            # delete, line 96
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 99
            # call R2, line 99
            if not self.r_R2():
                return False
            # delete, line 99
            if not self.slice_del():
                return False

            # try, line 100
            v_1 = self.limit - self.cursor
            try:
                # (, line 100
                # [, line 100
                self.ket = self.cursor
                # literal, line 100
                if not self.eq_s_b(2, u"ic"):
                    self.cursor = self.limit - v_1
                    raise lab0()
                # ], line 100
                self.bra = self.cursor
                # or, line 100
                try:
                    v_2 = self.limit - self.cursor
                    try:
                        # (, line 100
                        # call R2, line 100
                        if not self.r_R2():
                            raise lab2()
                        # delete, line 100
                        if not self.slice_del():
                            return False

                        raise lab1()
                    except lab2: pass
                    self.cursor = self.limit - v_2
                    # <-, line 100
                    if not self.slice_from(u"iqU"):
                        return False
                except lab1: pass
            except lab0: pass
        elif among_var == 3:
            # (, line 104
            # call R2, line 104
            if not self.r_R2():
                return False
            # <-, line 104
            if not self.slice_from(u"log"):
                return False
        elif among_var == 4:
            # (, line 107
            # call R2, line 107
            if not self.r_R2():
                return False
            # <-, line 107
            if not self.slice_from(u"u"):
                return False
        elif among_var == 5:
            # (, line 110
            # call R2, line 110
            if not self.r_R2():
                return False
            # <-, line 110
            if not self.slice_from(u"ent"):
                return False
        elif among_var == 6:
            # (, line 113
            # call RV, line 114
            if not self.r_RV():
                return False
            # delete, line 114
            if not self.slice_del():
                return False

            # try, line 115
            v_3 = self.limit - self.cursor
            try:
                # (, line 115
                # [, line 116
                self.ket = self.cursor
                # substring, line 116
                among_var = self.find_among_b(FrenchStemmer.a_2, 6)
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab3()
                # ], line 116
                self.bra = self.cursor
                if among_var == 0:
                    self.cursor = self.limit - v_3
                    raise lab3()
                elif among_var == 1:
                    # (, line 117
                    # call R2, line 117
                    if not self.r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    # delete, line 117
                    if not self.slice_del():
                        return False

                    # [, line 117
                    self.ket = self.cursor
                    # literal, line 117
                    if not self.eq_s_b(2, u"at"):
                        self.cursor = self.limit - v_3
                        raise lab3()
                    # ], line 117
                    self.bra = self.cursor
                    # call R2, line 117
                    if not self.r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    # delete, line 117
                    if not self.slice_del():
                        return False

                elif among_var == 2:
                    # (, line 118
                    # or, line 118
                    try:
                        v_4 = self.limit - self.cursor
                        try:
                            # (, line 118
                            # call R2, line 118
                            if not self.r_R2():
                                raise lab5()
                            # delete, line 118
                            if not self.slice_del():
                                return False

                            raise lab4()
                        except lab5: pass
                        self.cursor = self.limit - v_4
                        # (, line 118
                        # call R1, line 118
                        if not self.r_R1():
                            self.cursor = self.limit - v_3
                            raise lab3()
                        # <-, line 118
                        if not self.slice_from(u"eux"):
                            return False
                    except lab4: pass
                elif among_var == 3:
                    # (, line 120
                    # call R2, line 120
                    if not self.r_R2():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    # delete, line 120
                    if not self.slice_del():
                        return False

                elif among_var == 4:
                    # (, line 122
                    # call RV, line 122
                    if not self.r_RV():
                        self.cursor = self.limit - v_3
                        raise lab3()
                    # <-, line 122
                    if not self.slice_from(u"i"):
                        return False
            except lab3: pass
        elif among_var == 7:
            # (, line 128
            # call R2, line 129
            if not self.r_R2():
                return False
            # delete, line 129
            if not self.slice_del():
                return False

            # try, line 130
            v_5 = self.limit - self.cursor
            try:
                # (, line 130
                # [, line 131
                self.ket = self.cursor
                # substring, line 131
                among_var = self.find_among_b(FrenchStemmer.a_3, 3)
                if among_var == 0:
                    self.cursor = self.limit - v_5
                    raise lab6()
                # ], line 131
                self.bra = self.cursor
                if among_var == 0:
                    self.cursor = self.limit - v_5
                    raise lab6()
                elif among_var == 1:
                    # (, line 132
                    # or, line 132
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            # (, line 132
                            # call R2, line 132
                            if not self.r_R2():
                                raise lab8()
                            # delete, line 132
                            if not self.slice_del():
                                return False

                            raise lab7()
                        except lab8: pass
                        self.cursor = self.limit - v_6
                        # <-, line 132
                        if not self.slice_from(u"abl"):
                            return False
                    except lab7: pass
                elif among_var == 2:
                    # (, line 133
                    # or, line 133
                    try:
                        v_7 = self.limit - self.cursor
                        try:
                            # (, line 133
                            # call R2, line 133
                            if not self.r_R2():
                                raise lab10()
                            # delete, line 133
                            if not self.slice_del():
                                return False

                            raise lab9()
                        except lab10: pass
                        self.cursor = self.limit - v_7
                        # <-, line 133
                        if not self.slice_from(u"iqU"):
                            return False
                    except lab9: pass
                elif among_var == 3:
                    # (, line 134
                    # call R2, line 134
                    if not self.r_R2():
                        self.cursor = self.limit - v_5
                        raise lab6()
                    # delete, line 134
                    if not self.slice_del():
                        return False

            except lab6: pass
        elif among_var == 8:
            # (, line 140
            # call R2, line 141
            if not self.r_R2():
                return False
            # delete, line 141
            if not self.slice_del():
                return False

            # try, line 142
            v_8 = self.limit - self.cursor
            try:
                # (, line 142
                # [, line 142
                self.ket = self.cursor
                # literal, line 142
                if not self.eq_s_b(2, u"at"):
                    self.cursor = self.limit - v_8
                    raise lab11()
                # ], line 142
                self.bra = self.cursor
                # call R2, line 142
                if not self.r_R2():
                    self.cursor = self.limit - v_8
                    raise lab11()
                # delete, line 142
                if not self.slice_del():
                    return False

                # [, line 142
                self.ket = self.cursor
                # literal, line 142
                if not self.eq_s_b(2, u"ic"):
                    self.cursor = self.limit - v_8
                    raise lab11()
                # ], line 142
                self.bra = self.cursor
                # or, line 142
                try:
                    v_9 = self.limit - self.cursor
                    try:
                        # (, line 142
                        # call R2, line 142
                        if not self.r_R2():
                            raise lab13()
                        # delete, line 142
                        if not self.slice_del():
                            return False

                        raise lab12()
                    except lab13: pass
                    self.cursor = self.limit - v_9
                    # <-, line 142
                    if not self.slice_from(u"iqU"):
                        return False
                except lab12: pass
            except lab11: pass
        elif among_var == 9:
            # (, line 144
            # <-, line 144
            if not self.slice_from(u"eau"):
                return False
        elif among_var == 10:
            # (, line 145
            # call R1, line 145
            if not self.r_R1():
                return False
            # <-, line 145
            if not self.slice_from(u"al"):
                return False
        elif among_var == 11:
            # (, line 147
            # or, line 147
            try:
                v_10 = self.limit - self.cursor
                try:
                    # (, line 147
                    # call R2, line 147
                    if not self.r_R2():
                        raise lab15()
                    # delete, line 147
                    if not self.slice_del():
                        return False

                    raise lab14()
                except lab15: pass
                self.cursor = self.limit - v_10
                # (, line 147
                # call R1, line 147
                if not self.r_R1():
                    return False
                # <-, line 147
                if not self.slice_from(u"eux"):
                    return False
            except lab14: pass
        elif among_var == 12:
            # (, line 150
            # call R1, line 150
            if not self.r_R1():
                return False
            if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
                return False
            # delete, line 150
            if not self.slice_del():
                return False

        elif among_var == 13:
            # (, line 155
            # call RV, line 155
            if not self.r_RV():
                return False
            # fail, line 155
            # (, line 155
            # <-, line 155
            if not self.slice_from(u"ant"):
                return False
            return False
        elif among_var == 14:
            # (, line 156
            # call RV, line 156
            if not self.r_RV():
                return False
            # fail, line 156
            # (, line 156
            # <-, line 156
            if not self.slice_from(u"ent"):
                return False
            return False
        elif among_var == 15:
            # (, line 158
            # test, line 158
            v_11 = self.limit - self.cursor
            # (, line 158
            if not self.in_grouping_b(FrenchStemmer.g_v, 97, 251):
                return False
            # call RV, line 158
            if not self.r_RV():
                return False
            self.cursor = self.limit - v_11
            # fail, line 158
            # (, line 158
            # delete, line 158
            if not self.slice_del():
                return False

            return False
        return True

    def r_i_verb_suffix(self):
        # setlimit, line 163
        v_1 = self.limit - self.cursor
        # tomark, line 163
        if self.cursor < self.I_pV:
            return False
        self.cursor = self.I_pV
        v_2 = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit - v_1
        # (, line 163
        # [, line 164
        self.ket = self.cursor
        # substring, line 164
        among_var = self.find_among_b(FrenchStemmer.a_5, 35)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        # ], line 164
        self.bra = self.cursor
        if among_var == 0:
            self.limit_backward = v_2
            return False
        elif among_var == 1:
            # (, line 170
            if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
                self.limit_backward = v_2
                return False
            # delete, line 170
            if not self.slice_del():
                return False

        self.limit_backward = v_2
        return True

    def r_verb_suffix(self):
        # setlimit, line 174
        v_1 = self.limit - self.cursor
        # tomark, line 174
        if self.cursor < self.I_pV:
            return False
        self.cursor = self.I_pV
        v_2 = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit - v_1
        # (, line 174
        # [, line 175
        self.ket = self.cursor
        # substring, line 175
        among_var = self.find_among_b(FrenchStemmer.a_6, 38)
        if among_var == 0:
            self.limit_backward = v_2
            return False
        # ], line 175
        self.bra = self.cursor
        if among_var == 0:
            self.limit_backward = v_2
            return False
        elif among_var == 1:
            # (, line 177
            # call R2, line 177
            if not self.r_R2():
                self.limit_backward = v_2
                return False
            # delete, line 177
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 185
            # delete, line 185
            if not self.slice_del():
                return False

        elif among_var == 3:
            # (, line 190
            # delete, line 190
            if not self.slice_del():
                return False

            # try, line 191
            v_3 = self.limit - self.cursor
            try:
                # (, line 191
                # [, line 191
                self.ket = self.cursor
                # literal, line 191
                if not self.eq_s_b(1, u"e"):
                    self.cursor = self.limit - v_3
                    raise lab0()
                # ], line 191
                self.bra = self.cursor
                # delete, line 191
                if not self.slice_del():
                    return False

            except lab0: pass
        self.limit_backward = v_2
        return True

    def r_residual_suffix(self):
        # (, line 198
        # try, line 199
        v_1 = self.limit - self.cursor
        try:
            # (, line 199
            # [, line 199
            self.ket = self.cursor
            # literal, line 199
            if not self.eq_s_b(1, u"s"):
                self.cursor = self.limit - v_1
                raise lab0()
            # ], line 199
            self.bra = self.cursor
            # test, line 199
            v_2 = self.limit - self.cursor
            if not self.out_grouping_b(FrenchStemmer.g_keep_with_s, 97, 232):
                self.cursor = self.limit - v_1
                raise lab0()
            self.cursor = self.limit - v_2
            # delete, line 199
            if not self.slice_del():
                return False

        except lab0: pass
        # setlimit, line 200
        v_3 = self.limit - self.cursor
        # tomark, line 200
        if self.cursor < self.I_pV:
            return False
        self.cursor = self.I_pV
        v_4 = self.limit_backward
        self.limit_backward = self.cursor
        self.cursor = self.limit - v_3
        # (, line 200
        # [, line 201
        self.ket = self.cursor
        # substring, line 201
        among_var = self.find_among_b(FrenchStemmer.a_7, 7)
        if among_var == 0:
            self.limit_backward = v_4
            return False
        # ], line 201
        self.bra = self.cursor
        if among_var == 0:
            self.limit_backward = v_4
            return False
        elif among_var == 1:
            # (, line 202
            # call R2, line 202
            if not self.r_R2():
                self.limit_backward = v_4
                return False
            # or, line 202
            try:
                v_5 = self.limit - self.cursor
                try:
                    # literal, line 202
                    if not self.eq_s_b(1, u"s"):
                        raise lab2()
                    raise lab1()
                except lab2: pass
                self.cursor = self.limit - v_5
                # literal, line 202
                if not self.eq_s_b(1, u"t"):
                    self.limit_backward = v_4
                    return False
            except lab1: pass
            # delete, line 202
            if not self.slice_del():
                return False

        elif among_var == 2:
            # (, line 204
            # <-, line 204
            if not self.slice_from(u"i"):
                return False
        elif among_var == 3:
            # (, line 205
            # delete, line 205
            if not self.slice_del():
                return False

        elif among_var == 4:
            # (, line 206
            # literal, line 206
            if not self.eq_s_b(2, u"gu"):
                self.limit_backward = v_4
                return False
            # delete, line 206
            if not self.slice_del():
                return False

        self.limit_backward = v_4
        return True

    def r_un_double(self):
        # (, line 211
        # test, line 212
        v_1 = self.limit - self.cursor
        # among, line 212
        if self.find_among_b(FrenchStemmer.a_8, 5) == 0:
            return False
        self.cursor = self.limit - v_1
        # [, line 212
        self.ket = self.cursor
        # next, line 212
        if self.cursor <= self.limit_backward:
            return False
        self.cursor -= 1
        # ], line 212
        self.bra = self.cursor
        # delete, line 212
        if not self.slice_del():
            return False

        return True

    def r_un_accent(self):
        # (, line 215
        # atleast, line 216
        v_1 = 1
        # atleast, line 216
        try:
            while True:
                try:
                    try:
                        if not self.out_grouping_b(FrenchStemmer.g_v, 97, 251):
                            raise lab2()
                        v_1 -= 1
                        raise lab1()
                    except lab2: pass
                    raise lab0()
                except lab1: pass
        except lab0: pass
        if v_1 > 0:
            return False
        # [, line 217
        self.ket = self.cursor
        # or, line 217
        try:
            v_3 = self.limit - self.cursor
            try:
                # literal, line 217
                if not self.eq_s_b(1, u"\u00E9"):
                    raise lab4()
                raise lab3()
            except lab4: pass
            self.cursor = self.limit - v_3
            # literal, line 217
            if not self.eq_s_b(1, u"\u00E8"):
                return False
        except lab3: pass
        # ], line 217
        self.bra = self.cursor
        # <-, line 217
        if not self.slice_from(u"e"):
            return False
        return True

    def _stem(self):
        # (, line 221
        # do, line 223
        v_1 = self.cursor
        try:
            # call prelude, line 223
            if not self.r_prelude():
                raise lab0()
        except lab0: pass
        self.cursor = v_1
        # do, line 224
        v_2 = self.cursor
        try:
            # call mark_regions, line 224
            if not self.r_mark_regions():
                raise lab1()
        except lab1: pass
        self.cursor = v_2
        # backwards, line 225
        self.limit_backward = self.cursor
        self.cursor = self.limit
        # (, line 225
        # do, line 227
        v_3 = self.limit - self.cursor
        try:
            # (, line 227
            # or, line 237
            try:
                v_4 = self.limit - self.cursor
                try:
                    # (, line 228
                    # and, line 233
                    v_5 = self.limit - self.cursor
                    # (, line 229
                    # or, line 229
                    try:
                        v_6 = self.limit - self.cursor
                        try:
                            # call standard_suffix, line 229
                            if not self.r_standard_suffix():
                                raise lab6()
                            raise lab5()
                        except lab6: pass
                        self.cursor = self.limit - v_6
                        try:
                            # call i_verb_suffix, line 230
                            if not self.r_i_verb_suffix():
                                raise lab7()
                            raise lab5()
                        except lab7: pass
                        self.cursor = self.limit - v_6
                        # call verb_suffix, line 231
                        if not self.r_verb_suffix():
                            raise lab4()
                    except lab5: pass
                    self.cursor = self.limit - v_5
                    # try, line 234
                    v_7 = self.limit - self.cursor
                    try:
                        # (, line 234
                        # [, line 234
                        self.ket = self.cursor
                        # or, line 234
                        try:
                            v_8 = self.limit - self.cursor
                            try:
                                # (, line 234
                                # literal, line 234
                                if not self.eq_s_b(1, u"Y"):
                                    raise lab10()
                                # ], line 234
                                self.bra = self.cursor
                                # <-, line 234
                                if not self.slice_from(u"i"):
                                    return False
                                raise lab9()
                            except lab10: pass
                            self.cursor = self.limit - v_8
                            # (, line 235
                            # literal, line 235
                            if not self.eq_s_b(1, u"\u00E7"):
                                self.cursor = self.limit - v_7
                                raise lab8()
                            # ], line 235
                            self.bra = self.cursor
                            # <-, line 235
                            if not self.slice_from(u"c"):
                                return False
                        except lab9: pass
                    except lab8: pass
                    raise lab3()
                except lab4: pass
                self.cursor = self.limit - v_4
                # call residual_suffix, line 238
                if not self.r_residual_suffix():
                    raise lab2()
            except lab3: pass
        except lab2: pass
        self.cursor = self.limit - v_3
        # do, line 243
        v_9 = self.limit - self.cursor
        try:
            # call un_double, line 243
            if not self.r_un_double():
                raise lab11()
        except lab11: pass
        self.cursor = self.limit - v_9
        # do, line 244
        v_10 = self.limit - self.cursor
        try:
            # call un_accent, line 244
            if not self.r_un_accent():
                raise lab12()
        except lab12: pass
        self.cursor = self.limit - v_10
        self.cursor = self.limit_backward
        # do, line 246
        v_11 = self.cursor
        try:
            # call postlude, line 246
            if not self.r_postlude():
                raise lab13()
        except lab13: pass
        self.cursor = v_11
        return True

    def equals(self, o):
        return isinstance(o, FrenchStemmer)

    def hashCode(self):
        return hash("FrenchStemmer")
class lab0(BaseException): pass
class lab1(BaseException): pass
class lab2(BaseException): pass
class lab3(BaseException): pass
class lab4(BaseException): pass
class lab5(BaseException): pass
class lab6(BaseException): pass
class lab7(BaseException): pass
class lab8(BaseException): pass
class lab9(BaseException): pass
class lab10(BaseException): pass
class lab11(BaseException): pass
class lab12(BaseException): pass
class lab13(BaseException): pass
class lab14(BaseException): pass
class lab15(BaseException): pass
